#include <stdio.h>
#include <stdbool.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <pthread.h>
#include <time.h>
#include <sys/time.h>

/*struct timespec
{
    time_t tv_sec;       
    long int tv_nsec;      
};*/

#include "lib-mw-capture.h"
#include "wave-file.h"

#include "ProductVer.h"

#define MAX_AUDIO_INPUT_SOURCE_NUM 5
#define AUDIO_CAPTURE_SECONDS 10

static bool g_bAudioCapture = false;
//static bool g_bAudioRecord = false;
static bool g_bThreadExit = true;
static pthread_t g_pThreadID;

HCHANNEL g_hChannel = -1;

static void *aduio_capture_thread_proc(void *data)
{
    MWCAP_PTR hNotifyEvent = 0;
    HANDLE64 hNotify = 0;
    int nRet;

    do
    {
        if (g_hChannel == -1)
            break;

        hNotifyEvent = MWCreateEvent();
        if (hNotifyEvent == 0)
        {
            printf("Error: Create Notify event error\n");
            break;
        }

        hNotify = MWRegisterNotify(g_hChannel, hNotifyEvent, MWCAP_NOTIFY_AUDIO_FRAME_BUFFERED);
        if (hNotify == 0)
        {
            printf("Error: Register notify error\n");
            break;
        }

        ULONGLONG ullStatusBit = 0LL;
        MWCAP_AUDIO_SIGNAL_STATUS mass;
        if (MW_SUCCEEDED != MWGetAudioSignalStatus(g_hChannel, &mass))
            break;

        if (!mass.bLPCM)
            break;

        int nBitDepthInByte = mass.cBitsPerSample / 8;
        int nPresentChannel = 0;
        for (int i = 0; i < 4; i++)
        {
            bool bPresent  = (mass.wChannelValid & (0x01 << i)) ? true:false;
            nPresentChannel += bPresent ? 2 : 0;
        }

        WaveFileInit("./test.wav", mass.dwSampleRate, nPresentChannel, mass.cBitsPerSample);
        BYTE pbFrame[MWCAP_AUDIO_SAMPLES_PER_FRAME * nPresentChannel * nBitDepthInByte];

        while(!g_bThreadExit)
        {
            nRet = MWWaitEvent(hNotifyEvent, 1000);
            if (nRet <= 0)
                continue;

            if (MW_SUCCEEDED != MWGetNotifyStatus(g_hChannel, hNotify, &ullStatusBit))
                continue;

            if (ullStatusBit & MWCAP_NOTIFY_AUDIO_FRAME_BUFFERED)
            {

                MWCAP_AUDIO_CAPTURE_FRAME macf;

               while(1)
	       {
		   nRet = MWCaptureAudioFrame(g_hChannel, &macf);
		   if (MW_ENODATA == nRet)
		      break;
	       }

             //BYTE *pbSamples = (BYTE *)macf.adwSamples;


                for (int j = 0; j < (nPresentChannel/2); j++)

                {
                     for (int i = 0 ; i < MWCAP_AUDIO_SAMPLES_PER_FRAME; i++)
                    {

                         int nWritePos = (i * nPresentChannel + j * 2) * nBitDepthInByte;
                         int nReadPos = (i * MWCAP_AUDIO_MAX_NUM_CHANNELS + j);
                            int nReadPos2 = (i * MWCAP_AUDIO_MAX_NUM_CHANNELS + j + MWCAP_AUDIO_MAX_NUM_CHANNELS/2);

                     //for (int k = 0 ; k < nBitDepthInByte; k++)
                     //{
                         //pbFrame[nWritePos + k] = pbSamples[nReadPos + sizeof(DWORD) - nBitDepthInByte + k];
                         //pbFrame[nWritePos + k + nBitDepthInByte] = pbSamples[nReadPos2 + sizeof(DWORD) - nBitDepthInByte + k];
                         //pbFrame[nWritePos + k] = (BYTE)(macf.adwSamples[nReadPos + k] >> (32 - nBitDepthInByte * 8));
                     //}
                        DWORD dwLeft = macf.adwSamples[nReadPos] >> (32 - nBitDepthInByte * 8);
                        DWORD dwRight = macf.adwSamples[nReadPos2] >> (32 - nBitDepthInByte * 8);

                        memcpy(&pbFrame[nWritePos], &dwLeft, nBitDepthInByte);
                        memcpy(&pbFrame[nWritePos + nBitDepthInByte], &dwRight, nBitDepthInByte);
                    }
                }
		
		if (WaveFileIsOpen())
                	WaveFileWrite((const unsigned char *)pbFrame, MWCAP_AUDIO_SAMPLES_PER_FRAME * nPresentChannel * nBitDepthInByte);
            }
        }
        WaveFileExit();

    }while (false);

    if (hNotify!= 0)
    {
        MWUnregisterNotify(g_hChannel, hNotify);
        hNotify = 0;
    }

    if (hNotifyEvent != 0)
    {
        MWCloseEvent(hNotifyEvent);
        hNotifyEvent = 0;
    }

    return (void *)0;
}


int main(void)
{


    MWCaptureInitInstance();
    MWRefreshDevice();

    BYTE nMajor = 0;
    BYTE nMinor = 0;
    WORD nBuild = 0;
    MWGetVersion(&nMajor, &nMinor, &nBuild);
    printf("MWCapture SDK for Linux V%d.%d.%d - audio-capture\n", nMajor, nMinor, nBuild);

    MW_RESULT mr;
    do
    {
        int nChannelCount = MWGetChannelCount();
        if (nChannelCount == 0)
        {
            printf("No Prp Capture device is found\n");
            break;
        }

        MWCAP_CHANNEL_INFO channelInfo;
        mr = MWGetChannelInfoByIndex(0, &channelInfo);
        if(mr != MW_SUCCEEDED)
        {
            printf("Error: Get channel info failed\n");
            break;
        }

        g_hChannel = MWOpenChannel(channelInfo.byBoardIndex, channelInfo.byChannelIndex);
        if (g_hChannel == -1)
        {
            printf("Error: Open channel error\n");
            break;
        }

        printf("\nCapture channel info \n"
               "   %-13s : %d\n"
               "   %-13s : %d\n"
               "   %-13s : %s\n"
               "   %-13s : %s\n\n",
               "Board Index", channelInfo.byBoardIndex,
               "Channel Index", channelInfo.byChannelIndex,
               "Product Name", channelInfo.szProductName,
               "Serial Number", channelInfo.szBoardSerialNo);

        MWCAP_AUDIO_SIGNAL_STATUS status;
        mr = MWGetAudioSignalStatus(g_hChannel, &status);
        if(mr != MW_SUCCEEDED)
        {
            printf("Error: Get audio signal status failed\n");
            break;
        }

        int nChannels = 0;
        for (int i = 0; i < 4 ; i++)
        {
            if (status.wChannelValid & (0x01 < i))
                nChannels += 2;
        }

        if (nChannels == 0)
        {
            printf("Error:There is no audio signal\n");
            break;
        }
        else
            printf("Audio signal format:\n"
                   "   %-13s : %d\n"
                   "   %-13s : %d\n"
                   "   %-13s : %d\n"
                   "   %-13s : %s\n\n",
                   "Channels", nChannels,
                   "Samplerate", status.dwSampleRate,
                   "Bits", status.cBitsPerSample,
                   "LPCM", status.bLPCM ? "YES" : "NO"
                   );



        DWORD pdwInputSource[MAX_AUDIO_INPUT_SOURCE_NUM];
        DWORD dwInputCount = MAX_AUDIO_INPUT_SOURCE_NUM;
        mr = MWGetAudioInputSourceArray(g_hChannel, pdwInputSource, &dwInputCount);
        if (mr == MW_SUCCEEDED || dwInputCount != 0)
        {
            if (MWStartAudioCapture(g_hChannel) != MW_SUCCEEDED)
            {
                printf("Error: Start audio capture failed!\n ");
            }else {
                g_bAudioCapture = true;
                g_bThreadExit = false;
                if (pthread_create(&g_pThreadID, NULL, aduio_capture_thread_proc, NULL) != 0)
                {
                    fprintf(stderr, "thread create error: %m\n");
                    g_bThreadExit = true;
                }
                printf("Start capture audio data -- 0 s");
                fflush(stdout);
                //printf("Please Wait for 10 seconds and find the test.wav file on the current floder!\n");

                for (int i = 0; i < AUDIO_CAPTURE_SECONDS; i++)
                {
                    sleep(1);
                    printf("\rStart capture audio data -- %d s", i+1);
                    fflush(stdout);
                }
                printf("\n");
                printf("Stop capture audio data\n\n"
                       "Audio data is saved in test.wav!\n");

                g_bThreadExit = true;
                pthread_join(g_pThreadID, NULL);

                MWStopAudioCapture(g_hChannel);
                g_bAudioCapture = false;
            }
        }
        if (g_hChannel != -1)
        {
            MWCloseChannel(g_hChannel);
            g_hChannel = -1;
        }

    }while(false);

    MWCaptureExitInstance();

    //printf("\nPress Enter to exit!");
    //fflush(stdout);
    //getchar();

    return 0;

}

