#include <stdio.h>
#include <stdlib.h>
//#include <curses.h>
#include <termios.h>
#include "ProductVer.h"
#include "lib-mw-capture.h"

#define AUDIO_INPUT_TYPE_NUM 5 //The number of audio input type


static void PrintAudioInputSourceArray(DWORD dwAudioInputSources[], DWORD dwAudioInputSourceCount, DWORD dwCurType, BOOLEAN bAutoLink)
{
    if (dwAudioInputSourceCount == 0)
        return;

    printf("Audio Input :\n"
           "%c [a] : AutoLink\n", bAutoLink ? '*' : ' ');

    for (int i = 0; i < dwAudioInputSourceCount; i++)
    {
        int type = (int)(dwAudioInputSources[i] >> 8);
        switch (type)
        {
        case MWCAP_AUDIO_INPUT_TYPE_HDMI:
            printf("%c [%d] : HDMI\n", (dwCurType == type && !bAutoLink) ? '*' : ' ', i);
            break;
        case MWCAP_AUDIO_INPUT_TYPE_SDI:
            printf("%c [%d] : SDI\n", (dwCurType == type && !bAutoLink) ? '*' : ' ', i);
            break;
        case MWCAP_AUDIO_INPUT_TYPE_LINE_IN:
            printf("%c [%d] : LINE_IN\n", (dwCurType == type && !bAutoLink) ? '*' : ' ', i);
            break;
        case MWCAP_AUDIO_INPUT_TYPE_MIC_IN:
            printf("%c [%d] : MIC_IN\n", (dwCurType == type && !bAutoLink) ? '*' : ' ', i);
            break;
        default:
            break;
        }
    }
}

static void GetInputTypeString(BOOLEAN bAudoLink, DWORD dwInputType, char *szInputType)
{
    if (szInputType == NULL)
        return;

    if(bAudoLink)
    {
        strcpy(szInputType, "auto");
        return;
    }

    switch(dwInputType)
    {
    case MWCAP_AUDIO_INPUT_TYPE_HDMI:
        strcpy(szInputType, "HDMI");
        break;
    case MWCAP_AUDIO_INPUT_TYPE_SDI:
        strcpy(szInputType, "SDI");
        break;
    case MWCAP_AUDIO_INPUT_TYPE_LINE_IN:
        strcpy(szInputType, "LINE_IN");
        break;
    case MWCAP_AUDIO_INPUT_TYPE_MIC_IN:
        strcpy(szInputType, "MIC_IN");
        break;
    default:
        break;
    }

    return;
}

static int mygetch(void)
{
    struct termios oldt,
    newt;
    int ch;
    tcgetattr( STDIN_FILENO, &oldt );
    newt = oldt;
    newt.c_lflag &= ~( ICANON | ECHO );
    tcsetattr( STDIN_FILENO, TCSANOW, &newt );
    ch = getchar();
    tcsetattr( STDIN_FILENO, TCSANOW, &oldt );
    return ch;
}



int main(void)
{
    MWCaptureInitInstance();
    MWRefreshDevice();

    BYTE nMajor = 0;
    BYTE nMinor = 0;
    WORD nBuild = 0;
    MWGetVersion(&nMajor, &nMinor, &nBuild);
    printf("MWCapture SDK for Linux V%d.%d.%d - audio-capture\n", nMajor, nMinor, nBuild);

    MW_RESULT mr;
    HCHANNEL hChannel = -1;
    do
    {
        int nChannelCount = MWGetChannelCount();
        if (nChannelCount == 0)
        {
            printf("No Pro Capture device is found\n");
            break;
        }

        MWCAP_CHANNEL_INFO channelInfo;
        mr = MWGetChannelInfoByIndex(0, &channelInfo);
        if(mr != MW_SUCCEEDED)
        {
            printf("Error: Get channel info error\n");
            break;
        }

        hChannel = MWOpenChannel(channelInfo.byBoardIndex, channelInfo.byChannelIndex);
        if (hChannel == -1)
        {
            printf("Error: Open channel error\n");
            break;
        }

        printf("\nCapture channel info \n"
               "   %-13s : %d\n"
               "   %-13s : %d\n"
               "   %-13s : %s\n"
               "   %-13s : %s\n\n",
               "Board Index", channelInfo.byBoardIndex,
               "Channel Index", channelInfo.byChannelIndex,
               "Product Name", channelInfo.szProductName,
               "Serial Number", channelInfo.szBoardSerialNo);

        DWORD dwCurSource;
        mr = MWGetAudioInputSource(hChannel, &dwCurSource);
        if (mr != MW_SUCCEEDED)
        {
            printf("Error: Get audio input source error\n");
            break;
        }

        BOOLEAN bLink;
        mr = MWGetAVInputSourceLink(hChannel, &bLink);
        if (mr != MW_SUCCEEDED)
        {
            printf("Error: Get AV input source link error\n");
            break;
        }

        char szInputType[32];
        GetInputTypeString(bLink, INPUT_TYPE(dwCurSource), szInputType);

        if(bLink)
            printf("AutoLink = YES\n");
        else
            printf("AutoLink = NO, Input Type = %s\n",
                   szInputType);

        DWORD dwAudioInputSources[AUDIO_INPUT_TYPE_NUM];
        DWORD dwAudioInputSourceCount = AUDIO_INPUT_TYPE_NUM;
        mr = MWGetAudioInputSourceArray(hChannel, dwAudioInputSources, &dwAudioInputSourceCount);
        if(mr != MW_SUCCEEDED)
        {
            printf("Error: Get audio input sources arrray error\n");
            break;
        }


        PrintAudioInputSourceArray(dwAudioInputSources, dwAudioInputSourceCount, INPUT_TYPE(dwCurSource), bLink);

        printf("\nSelected audio input source  'a' or (0 - %d)  ", dwAudioInputSourceCount -1);
        fflush(stdout);

        char c = 0;
        do
        {
            c = mygetch();
            printf("\b%c", c);
            fflush(stdout);
        } while( (c < '0' || c > '0' + dwAudioInputSourceCount -1) && c != 'a');
        printf("\n\n");

        int nIndex = (int)(c - '0');
        if (c == 'a')
        {
            mr = MWSetAVInputSourceLink(hChannel, TRUE);
            if (mr != MW_SUCCEEDED)
            {
                printf("Error: Set AV input source link error\n");
                break;
            }
        }
        else
        {
            mr = MWSetAVInputSourceLink(hChannel, FALSE);
            if (mr != MW_SUCCEEDED)
            {
                printf("Error: Set AV input source link error\n");
                break;
            }

            mr = MWSetAudioInputSource(hChannel, dwAudioInputSources[nIndex]);
            if (mr != MW_SUCCEEDED)
            {
                printf("Error: Set audio input source error\n");
                break;
            }
        }



        mr = MWGetAudioInputSource(hChannel, &dwCurSource);
        if (mr != MW_SUCCEEDED)
        {
            printf("Error: Get audio input source error\n");
            break;
        }


        mr = MWGetAVInputSourceLink(hChannel, &bLink);
        if (mr != MW_SUCCEEDED)
        {
            printf("Error: Get AV input source link error\n");
            break;
        }

        GetInputTypeString(bLink, INPUT_TYPE(dwCurSource), szInputType);
        printf("Set audio input type to %s successfully\n",
               szInputType);

        //PrintAudioInputSourceArray(dwAudioInputSources, dwAudioInputSourceCount, INPUT_TYPE(dwCurSource), bLink);

    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;

    }

    MWCaptureExitInstance();


    return 0;
}
