#include "cgroupbox.h"
#include <QPixmap>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QPaintEvent>
#include <QPainter>

CGroupBox::CGroupBox(QWidget *parent) :
    QGroupBox(parent)
{
    setFixedSize(1200, 320);

    m_cmbDevices = new QComboBox(this);
    m_cmbDevices->setGeometry(802, 0, 200, 25);

    m_mwCapture = new MWCapture;
    m_mwCapture->SetCallback(this);

    createDeviceComboBox();

    m_lblPic = new QLabel;
    m_lblPic->setFixedHeight(270);

    QGridLayout *midGidLayout = new QGridLayout;
    int nIndex = 0;
    m_lblSignalStatusTitle = new QLabel;
    m_lblSignalStatusTitle->setText(tr("Input signal status:"));
    m_lblSignalStatus = new QLabel;
    m_lblSignalStatus->setText(tr("-"));
    midGidLayout->addWidget(m_lblSignalStatusTitle, nIndex, 0);
    midGidLayout->addWidget(m_lblSignalStatus, nIndex++, 1);

    m_lblResolutionTitle = new QLabel;
    m_lblResolutionTitle->setText(tr("Resolution:"));
    m_lblResolution = new QLabel;
    m_lblResolution->setText(tr("-"));
    midGidLayout->addWidget(m_lblResolutionTitle, nIndex, 0);
    midGidLayout->addWidget(m_lblResolution, nIndex++, 1);


    m_lblTotalScanSizeTitle = new QLabel;
    m_lblTotalScanSizeTitle->setText(tr("Total scan size:"));
    m_lblTotalScanSize = new QLabel;
    m_lblTotalScanSize->setText(tr("-"));
    midGidLayout->addWidget(m_lblTotalScanSizeTitle, nIndex, 0);
    midGidLayout->addWidget(m_lblTotalScanSize, nIndex++, 1);

    m_lblActiveAreaOffsetTitle = new QLabel;
    m_lblActiveAreaOffsetTitle->setText(tr("Active area offset:"));
    m_lblActiveAreaOffset = new QLabel;
    m_lblActiveAreaOffset->setText(tr("-"));
    midGidLayout->addWidget(m_lblActiveAreaOffsetTitle, nIndex, 0);
    midGidLayout->addWidget(m_lblActiveAreaOffset, nIndex++, 1);


    m_lblCaptureFormatTitle = new QLabel;
    m_lblCaptureFormatTitle->setText(tr("Capture format:"));
    m_lblCaptureFormat = new QLabel;
    m_lblCaptureFormat->setText(tr("-"));
    midGidLayout->addWidget(m_lblCaptureFormatTitle, nIndex, 0);
    midGidLayout->addWidget(m_lblCaptureFormat, nIndex++, 1);


    m_lblCaptureLatencyTitle = new QLabel;
    m_lblCaptureLatencyTitle->setText(tr("Capture latency:"));
    m_lblCaptureLatency = new QLabel;
    m_lblCaptureLatency->setText(tr("-"));
    midGidLayout->addWidget(m_lblCaptureLatencyTitle, nIndex, 0);
    midGidLayout->addWidget(m_lblCaptureLatency, nIndex++, 1);


    m_lblTotalLatencyTitle = new QLabel;
    m_lblTotalLatencyTitle->setText(tr("Total latency:"));
    m_lblTotalLatency = new QLabel;
    m_lblTotalLatency->setText(tr("-"));
    midGidLayout->addWidget(m_lblTotalLatencyTitle, nIndex, 0);
    midGidLayout->addWidget(m_lblTotalLatency, nIndex++, 1);

    m_wndPreview = new QWidget;
    m_wndPreview->setObjectName(tr("PreviewWidget"));
    m_wndPreview->setFixedHeight(275);

    QHBoxLayout *mainLayout = new QHBoxLayout;
    mainLayout->addWidget(m_lblPic,1);
    mainLayout->addLayout(midGidLayout,1);
    mainLayout->addWidget(m_wndPreview,1);

    setLayout(mainLayout);

    m_bLowLatency = FALSE;

    m_xcbRenderer = new VideoRenderXCB;
    m_xcbRenderer->Init(0, 0, m_wndPreview->winId());
    m_xcbRenderer->SetFrameFormat(1920, 1080, MWFOURCC_YUY2, false);

    m_timerLatencyInfo = new QTimer;

    connect(m_cmbDevices, SIGNAL(currentIndexChanged(int)), this, SLOT(onCurrentIndexChanged(int)));
    connect(m_timerLatencyInfo, SIGNAL(timeout()), this, SLOT(updateLatencyInfo()));

}

CGroupBox::~CGroupBox()
{
    delete m_mwCapture;
    m_mwCapture = NULL;

    delete m_xcbRenderer;
    m_xcbRenderer = NULL;
}

void CGroupBox::setPicture(QString strPicName)
{
    QPixmap *pixmap = new QPixmap(strPicName);
    m_lblPic->setPixmap(pixmap->scaled(396, 270, Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
}

void CGroupBox::enableLowLatency(BOOL bLowLatency)
{
    m_bLowLatency = bLowLatency;
}

void CGroupBox::VideoCaptureCallback(MWCAP_PTR pbImage, unsigned int cbImage, void *pvParam)
{
    if (m_xcbRenderer != NULL)
    {
        m_xcbRenderer ->DisplayFrame((void *)(unsigned long)pbImage, cbImage);
    }
    return;
}


void CGroupBox::VideoSignalChangedCallback()
{
    updateVideoSignalStatus();
    MWCAP_VIDEO_SIGNAL_STATUS signalStatus;
    BOOL bRet = m_mwCapture->GetVideoSignalStatus(&signalStatus);

    if (!bRet || signalStatus.state == MWCAP_VIDEO_SIGNAL_NONE)
    {
        m_timerLatencyInfo->stop();
        updateLatencyInfo();
    }
    else
    {
        m_timerLatencyInfo->start();
    }


    return;
}


// Protect
/*void CGroupBox::paintEvent(QPaintEvent *event)
{
    if (m_bRefreshPreviewWin)
    {
        QPainter painter(this);
        painter.setBrush(Qt::black);
        QRect rect = m_wndPreview->geometry();
        painter.drawRect(rect);

        m_bRefreshPreviewWin = false;
    }

    event->accept();
}*/
// Slots
void CGroupBox::onCurrentIndexChanged(int nIndex)
{
    if(nIndex == 0)
    {
        m_mwCapture->CloseChannel();
        updateVideoSignalStatus();
        //m_bRefreshPreviewWin = true;
        update();
        m_timerLatencyInfo->stop();
        updateVideoSignalStatus();
        updateLatencyInfo();
    }
    else
    {
        m_mwCapture->CloseChannel();
        m_mwCapture->OpenChannel(nIndex - 1, DEFAULT_WIDTH, DEFAULT_HEIGHT, DEFAULT_FOURCC, m_bLowLatency, 0);
        updateVideoSignalStatus();
        m_timerLatencyInfo->start(1000);
    }

    return;
}

void CGroupBox::updateVideoSignalStatus()
{
    MWCAP_VIDEO_SIGNAL_STATUS signalStatus;
    BOOL bRet = m_mwCapture->GetVideoSignalStatus(&signalStatus);

    if (!bRet || signalStatus.state == MWCAP_VIDEO_SIGNAL_NONE)
    {
        m_lblSignalStatus->setText(tr("-"));
        m_lblResolution->setText(tr("-"));
        m_lblTotalScanSize->setText(tr("-"));
        m_lblActiveAreaOffset->setText(tr("-"));
        m_lblCaptureFormat->setText(tr("-"));

        return;
    }

    switch (signalStatus.state)
    {
    case MWCAP_VIDEO_SIGNAL_LOCKING:
        m_lblSignalStatus->setText(tr("Locking"));
        break;
    case MWCAP_VIDEO_SIGNAL_LOCKED:
        m_lblSignalStatus->setText(tr("Locked"));
        break;
    case MWCAP_VIDEO_SIGNAL_UNSUPPORTED:
        m_lblSignalStatus->setText(tr("Unsupported"));
        break;
    default:
        m_lblSignalStatus->setText(tr("-"));
        break;
    }

    float fFps = 1.0*10000000/signalStatus.dwFrameDuration;
    m_lblResolution->setText(QString("%1x%2 %3FPS").arg(signalStatus.cx).arg(signalStatus.cy).arg(fFps, 0, 'f', 2));
    m_lblTotalScanSize->setText(QString("%1x%2").arg(signalStatus.cxTotal).arg(signalStatus.cyTotal));
    m_lblActiveAreaOffset->setText(QString("x:%1 y:%2").arg(signalStatus.x).arg((signalStatus.y)));
    m_lblCaptureFormat->setText(QString("1920x1080 YUY2 %1").arg(fFps, 0, 'f', 2));
}


void CGroupBox::updateLatencyInfo()
{
    MWCAP_VIDEO_SIGNAL_STATUS signalStatus;
    BOOL bRet = m_mwCapture->GetVideoSignalStatus(&signalStatus);

    if (!bRet || signalStatus.state == MWCAP_VIDEO_SIGNAL_NONE)
    {
        m_lblCaptureLatency->setText(tr("-"));
        m_lblTotalLatency->setText(tr("-"));

        return;
    }

    LONGLONG llCaptureTime, llLatencyTime, llFrameCount;
    m_mwCapture->GetStatistics(&llCaptureTime, &llLatencyTime, &llFrameCount);
    float fCaptureLatency = 1.0*llCaptureTime/10000/llFrameCount;
    float fTotalLatency = 1.0*llLatencyTime/10000/llFrameCount;

    m_lblCaptureLatency->setText(QString("%1 ms").arg(fCaptureLatency, 0, 'f', 2));
    m_lblTotalLatency->setText(QString("%1 ms").arg(fTotalLatency, 0, 'f', 2));

    return;
}


//Private
void CGroupBox::createDeviceComboBox()
{
    m_cmbDevices->addItem(tr("(No selected channel)"));

    if (m_mwCapture == NULL)
        return;

    int nChannelCount = m_mwCapture->GetChannelCount();
    for (int i = 0; i < nChannelCount; i++)
    {
        char szDeviceName[64];
        m_mwCapture->GetDeviceNameByIndex(i, szDeviceName);
        m_cmbDevices->addItem(QString(szDeviceName));
    }

    return;
}














