#include "settingsdlg.h"


CSettingsDlg::CSettingsDlg(QWidget *parent) : QDialog(parent)
{
    setWindowTitle(tr("Settings"));
    setMinimumSize(600,300);

    //params init
    m_nNumChannel = 0;
    memset(m_stChannelsInfo,0,sizeof(CHANNEL_INDEX)*MAX_CHANNEL_COUNT);
    m_nFormat = 1;
    memset(&m_stSize,0,sizeof(struct st_size));
    m_stSize.width = DEFAULT_WIDTH;
    m_stSize.height = DEFAULT_HEIGHT;
    m_dFPS = 59.94;
    m_mwCapture = new MWCapture;
    m_strSignalStatus = "--";
    m_strResolution = "--";

    //Layout
    //top1
    m_labFormat = new QLabel(tr("Format:"));
    m_cmbFormat = new QComboBox;
    m_cmbFormat->addItem("1x1");
    m_cmbFormat->addItem("1x2");
    m_cmbFormat->addItem("2x2");
    m_cmbFormat->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed);
    m_hblFormat = new QHBoxLayout();
    m_hblFormat->setAlignment(Qt::AlignLeft);
    m_hblFormat->addWidget(m_labFormat);
    m_hblFormat->addWidget(m_cmbFormat);
    //top2
    m_labSize = new QLabel(tr("Size:"));
    m_cmbSize = new QComboBox;
    m_cmbSize->addItem("1920x1080");
    m_cmbSize->addItem("1680x1050");
    m_cmbSize->addItem("1440x900");
    m_cmbSize->addItem("1368x768");
    m_cmbSize->addItem("1280x1024");
    m_cmbSize->addItem("1280x960");
    m_cmbSize->addItem("1280x800");
    m_cmbSize->addItem("1280x720");
    m_cmbSize->addItem("1024x768");
    m_cmbSize->addItem("1024x576");
    m_cmbSize->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed);
    m_hblSize = new QHBoxLayout();
    m_hblSize->setAlignment(Qt::AlignLeft);
    m_hblSize->addWidget(m_labSize);
    m_hblSize->addWidget(m_cmbSize);
    //top3
    m_labFPS = new QLabel(tr("FPS:"));
    m_cmbFPS = new QComboBox;
    m_cmbFPS->addItem("59.94");
    m_cmbFPS->addItem("50.00");
    m_cmbFPS->addItem("30.00");
    m_cmbFPS->addItem("29.97");
    m_cmbFPS->addItem("25.00");
    m_cmbFPS->addItem("20.00");
    m_cmbFPS->addItem("15.00");
    m_cmbFPS->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed);
    m_hblFPS = new QHBoxLayout();
    m_hblFPS->setAlignment(Qt::AlignLeft);
    m_hblFPS->addWidget(m_labFPS);
    m_hblFPS->addWidget(m_cmbFPS);
    //top-total
    m_hblTop = new QHBoxLayout();
    m_hblTop->addLayout(m_hblFormat);
    m_hblTop->addLayout(m_hblSize);
    m_hblTop->addLayout(m_hblFPS);
    //table
    m_table = new QTableWidget;
    //bottom1
    m_chboxSync = new QCheckBox;
    m_labSync = new QLabel;
    m_labSync->setText(tr("Sync Mode"));
    m_hblCheck = new QHBoxLayout;
    m_hblCheck->setAlignment(Qt::AlignLeft);
    m_hblCheck->addWidget(m_chboxSync);
    m_hblCheck->addWidget(m_labSync);
    //bottom2
    m_btnOK = new QPushButton;
    m_btnOK->setText(tr("OK"));
    m_btnOK->setMinimumSize(75,35);
    m_btnOK->setMaximumSize(75,35);
    m_btnCancel = new QPushButton;
    m_btnCancel->setText(tr("Cancel"));
    m_btnCancel->setMinimumSize(75,35);
    m_btnCancel->setMaximumSize(75,35);
    m_hblBtn = new QHBoxLayout();
    m_hblBtn->setSpacing(10);
    m_hblBtn->setAlignment(Qt::AlignRight);
    m_hblBtn->addWidget(m_btnOK);
    m_hblBtn->addWidget(m_btnCancel);
    //bottom-total
    m_hblBottom = new QHBoxLayout;
    m_hblBottom->addLayout(m_hblCheck);
    m_hblBottom->addLayout(m_hblBtn);
    //mainlayout
    m_vblMain = new QVBoxLayout;
    m_vblMain->addLayout(m_hblTop,1);
    m_vblMain->addWidget(m_table,1);
    m_vblMain->addLayout(m_hblBottom,1);
    setLayout(m_vblMain);

    createDeviceTable();
    initStatus();

    //SLOT:
    connect(m_cmbFormat,SIGNAL(currentIndexChanged(int)),this,SLOT(onCurrentFormatIndexChanged(int)));
    connect(m_cmbSize, SIGNAL(currentIndexChanged(int)), this, SLOT(onCurrentSizeIndexChanged(int)));
    connect(m_cmbFPS, SIGNAL(currentIndexChanged(int)), this, SLOT(onCurrentFPSIndexChanged(int)));
    connect(m_table,SIGNAL(cellClicked(int,int)),this,SLOT(onTableLineClick(int,int)));
    connect(m_btnOK,SIGNAL(clicked()),this,SLOT(onPushButtonClickOK()));
    connect(m_btnCancel,SIGNAL(clicked()),this,SLOT(onPushButtonClickCancel()));
}


CSettingsDlg::~CSettingsDlg()
{
    delete m_mwCapture;
    m_mwCapture = NULL;
}


void CSettingsDlg::createDeviceTable()
{
    if (m_mwCapture == NULL)
       return;

    int nChannelCount = m_mwCapture->GetChannelCount();
    if(nChannelCount == 0)
    {
        m_table->setRowCount(1);
        m_table->setItem(0,1,new QTableWidgetItem("No Device"));
        return;
    }
    m_table->setRowCount(nChannelCount);//n Row
    m_table->setColumnCount(3);//3 Col
    m_table->setColumnWidth(0,30);//Col-0 width
    m_table->setColumnWidth(1,220);//Col-1 width
    m_table->horizontalHeader()->setStretchLastSection(true);//Col-last adapt
    m_table->setEditTriggers(QAbstractItemView::NoEditTriggers);//Disable Edit
    m_table->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);//Disable H-Scroll
    QStringList header;
    header<<" "<<"Name"<<"Status";
    m_table->setHorizontalHeaderLabels(header);
    for (int i = 0; i < nChannelCount; i++)
    {
        char szDeviceName[64];
        m_mwCapture->GetDeviceNameByIndex(i, szDeviceName);
        m_table->setItem(i,1,new QTableWidgetItem(QString(szDeviceName)));

        QTableWidgetItem *check = new QTableWidgetItem();
        check->setCheckState(Qt::Unchecked);
        m_table->setItem(i,0,check);
    }
    m_table->item(0,0)->setCheckState(Qt::Checked);
    m_chboxSync->setCheckable(false);
    updateCheckBox();

    return;
}


void CSettingsDlg::initStatus()
{
    MW_RESULT mr;
    CHANNEL_INDEX stAllChannelIndex[MAX_CHANNEL_COUNT];
    int nChannelCount = m_mwCapture->GetChannelCount();

    for(int i=0;i<nChannelCount;i++)
    {
        MWCAP_CHANNEL_INFO channelInfo;
        if (MW_SUCCEEDED != MWGetChannelInfoByIndex(i,&channelInfo))
            continue;
        stAllChannelIndex[i].nBoard = channelInfo.byBoardIndex;
        stAllChannelIndex[i].nChannel = channelInfo.byChannelIndex;
    }

    m_mwCapture->Create(stAllChannelIndex,nChannelCount,MAX_CHANNEL_COUNT,MAX_CHANNEL_COUNT,DEFAULT_WIDTH,DEFAULT_HEIGHT,DEFAULT_FOURCC,59.94,FALSE,NULL,0);

    for(int i=0;i<nChannelCount;i++)
    {
        MWCAP_VIDEO_SIGNAL_STATUS signalStatus;
        mr = MWGetVideoSignalStatus(m_mwCapture->m_arrChannel[i].hChannel,&signalStatus);
        if(mr != MW_SUCCEEDED)
            continue;
        if(signalStatus.state == MWCAP_VIDEO_SIGNAL_NONE)
        {
            m_table->setItem(i,2,new QTableWidgetItem(QString("--,--")));
        }
        else
        {
            switch (signalStatus.state)
            {
            case MWCAP_VIDEO_SIGNAL_LOCKING:
                m_strSignalStatus = "Locking";
                break;
            case MWCAP_VIDEO_SIGNAL_LOCKED:
                m_strSignalStatus = "Locked";
                break;
            case MWCAP_VIDEO_SIGNAL_UNSUPPORTED:
                m_strSignalStatus = "Unsupported";
                break;
            default:
                m_strSignalStatus = "--";
                break;
            }
            float fFps = 1.0*10000000/signalStatus.dwFrameDuration;
            m_strResolution = QString("%1x%2,%3FPS").arg(signalStatus.cx).arg(signalStatus.cy).arg(fFps, 0, 'f', 2);
            m_table->setItem(i,2,new QTableWidgetItem(m_strSignalStatus + "," + m_strResolution));
        }
    }
    m_mwCapture->Destroy();
}


void CSettingsDlg::updateCheckBox()
{
    int nCheck = 0;
    int nChannelCount = m_mwCapture->GetChannelCount();

    for(int i=0;i<nChannelCount;i++)
    {
        if(m_table->item(i,0)->checkState() == Qt::Checked)
        {
             nCheck++;
             if(nCheck > 1)//CheckBox
             {
                m_chboxSync->setCheckable(true);
                m_chboxSync->setCheckState(Qt::Checked);
             }
             else
             {
                 m_chboxSync->setCheckable(false);
                 m_chboxSync->setCheckState(Qt::Unchecked);
             }

             if(nCheck >= m_nFormat)//Table Select
             {
                for(int j=0;j<nChannelCount;j++)
                {
                    if(m_table->item(j,0)->checkState() == Qt::Unchecked)
                        m_table->item(j,0)->setFlags(Qt::NoItemFlags);
                }
                return;
             }
        }
    }

    for(int k=0;k<nChannelCount;k++)//Unlock
    {
        m_table->item(k,0)->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsEnabled | Qt::ItemIsSelectable);
    }
}


void CSettingsDlg::onTableLineClick(int nRow,int nCol)
{
    if(nCol == 0)
    {
        updateCheckBox();
    }
    return;
}


void CSettingsDlg::onCurrentFormatIndexChanged(int nIndex)
{
    if(nIndex == 0)//1x1
    {
        m_nFormat = 1;
        m_chboxSync->setCheckable(false);
        m_chboxSync->setCheckState(Qt::Unchecked);
    }
    else if(nIndex == 1)//1x2
    {
        m_nFormat = 2;
        m_chboxSync->setCheckable(true);
        m_chboxSync->setCheckState(Qt::Checked);
    }
    else if(nIndex == 2)//2x2
    {
        m_nFormat = 4;
        m_chboxSync->setCheckable(true);
        m_chboxSync->setCheckState(Qt::Checked);
    }

    int nCount = 0;
    int nChannelCount = m_mwCapture->GetChannelCount();
    for(int i=0;i<nChannelCount;i++)
    {
        if(m_table->item(i,0)->checkState() == Qt::Checked)
            nCount++;
        if(nCount >= m_nFormat)
        {
            for(int j=i+1;j<nChannelCount;j++)
                m_table->item(j,0)->setCheckState(Qt::Unchecked);
            break;
        }
    }
    updateCheckBox();

    return;
}


void CSettingsDlg::onCurrentSizeIndexChanged(int nIndex)
{
    switch(nIndex)
    {
    case 0:
        m_stSize.width = 1920;
        m_stSize.height = 1080;//1920x1080
        break;
    case 1:
        m_stSize.width = 1680;
        m_stSize.height = 1050;//1680x1050
        break;
    case 2:
        m_stSize.width = 1440;
        m_stSize.height = 900;//1440x900
        break;
    case 3:
        m_stSize.width = 1368;
        m_stSize.height = 768;//1368x768
        break;
    case 4:
        m_stSize.width = 1280;
        m_stSize.height = 1024;//1280x1024
        break;
    case 5:
        m_stSize.width = 1280;
        m_stSize.height = 960;//1280x960
        break;
    case 6:
        m_stSize.width = 1280;
        m_stSize.height = 800;//1280x800
        break;
    case 7:
        m_stSize.width = 1280;
        m_stSize.height = 720;//1280x720
        break;
    case 8:
        m_stSize.width = 1024;
        m_stSize.height = 768;//1024x768
        break;
    case 9:
        m_stSize.width = 1024;
        m_stSize.height = 576;//1024x576
        break;
    default:
        m_stSize.width = DEFAULT_WIDTH;
        m_stSize.height = DEFAULT_HEIGHT;
        break;
    }

    return;
}


void CSettingsDlg::onCurrentFPSIndexChanged(int nIndex)
{
    switch(nIndex)
    {
    case 0:
        m_dFPS = 59.94;
        break;
    case 1:
        m_dFPS = 50.00;
        break;
    case 2:
        m_dFPS = 30.00;
        break;
    case 3:
        m_dFPS = 29.97;
        break;
    case 4:
        m_dFPS = 25.00;
        break;
    case 5:
        m_dFPS = 20.00;
        break;
    case 6:
        m_dFPS = 15.00;
        break;
    default:
        m_dFPS = 59.94;
        break;
    }

    return;
}


void CSettingsDlg::onPushButtonClickOK()
{
    int nChannelCount = m_mwCapture->GetChannelCount();
    m_nNumChannel = 0;
    memset(m_stChannelsInfo,0,sizeof(CHANNEL_INDEX)*MAX_CHANNEL_COUNT);

    for(int i=0;i<nChannelCount;i++)
    {
        if(m_table->item(i,0)->checkState() == Qt::Checked)
        {
            MWCAP_CHANNEL_INFO channelInfo;
            if(MW_SUCCEEDED != MWGetChannelInfoByIndex(i,&channelInfo))
                continue;
            m_stChannelsInfo[m_nNumChannel].nBoard = channelInfo.byBoardIndex;
            m_stChannelsInfo[m_nNumChannel++].nChannel = channelInfo.byChannelIndex;
        }
    }

    if(m_chboxSync->checkState() == Qt::Checked)
        emit infoSend(m_stChannelsInfo,m_nNumChannel,m_nFormat,m_stSize,m_dFPS,true);//send to father
    else
        emit infoSend(m_stChannelsInfo,m_nNumChannel,m_nFormat,m_stSize,m_dFPS,false);
}


void CSettingsDlg::onPushButtonClickCancel()
{
    hide();
}


