#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
//#include <curses.h>
#include <termios.h>
#include "ProductVer.h"
#include "lib-mw-capture.h"

#define VIDEO_INPUT_TYPE_NUM 7 //The number of video input type

void PrintVideoInputSourceArray(DWORD dwVideoInputSources[], DWORD dwVideoInputSourceCount, DWORD dwCurType, BOOLEAN bAutoScan)
{
    if (dwVideoInputSourceCount == 0)
        return;

    printf("Video Input : \n");
    printf("%c [a] : AutoScan\n", bAutoScan ? '*' : ' ');
    
    for (int i = 0; i < dwVideoInputSourceCount; i ++) {
		int type = INPUT_TYPE(dwVideoInputSources[i]);
		switch (type) {
			case MWCAP_VIDEO_INPUT_TYPE_HDMI:
                printf("%c [%d] : HDMI (0x%x)\n", (type == dwCurType && !bAutoScan) ? '*' : ' ', i, dwVideoInputSources[i] );
				break;
			case MWCAP_VIDEO_INPUT_TYPE_VGA:
                printf("%c [%d] : VGA (0x%x)\n", (type == dwCurType && !bAutoScan) ? '*' : ' ', i, dwVideoInputSources[i] );
				break;
			case MWCAP_VIDEO_INPUT_TYPE_SDI:
                printf("%c [%d] : SDI (0x%x)\n", (type == dwCurType && !bAutoScan) ? '*' : ' ', i, dwVideoInputSources[i] );
				break;
			case MWCAP_VIDEO_INPUT_TYPE_COMPONENT:
                printf("%c [%d] : Component (0x%x)\n", (type == dwCurType && !bAutoScan) ? '*' : ' ', i, dwVideoInputSources[i] );
				break;
			case MWCAP_VIDEO_INPUT_TYPE_CVBS:
                printf("%c [%d] : CVBS (0x%x)\n", (type == dwCurType && !bAutoScan) ? '*' : ' ', i, dwVideoInputSources[i] );
				break;
			case MWCAP_VIDEO_INPUT_TYPE_YC:
                printf("%c [%d] : YC (0x%x)\n", (type == dwCurType && !bAutoScan) ? '*' : ' ', i, dwVideoInputSources[i] );
				break;
		}
	}	
}

static int mygetch(void)
{
    struct termios oldt,
    newt;
    int ch;
    tcgetattr( STDIN_FILENO, &oldt );
    newt = oldt;
    newt.c_lflag &= ~( ICANON | ECHO );
    tcsetattr( STDIN_FILENO, TCSANOW, &newt );
    ch = getchar();
    tcsetattr( STDIN_FILENO, TCSANOW, &oldt );
    return ch;
}

static void GetInPutTypestring(BOOLEAN bAutoScan, DWORD dwType, char *szInputType)
{
    if (szInputType == NULL)
        return;

    if (bAutoScan)
    {
        strcpy(szInputType, "auto");
        return;
    }

    switch (dwType) {
    case MWCAP_VIDEO_INPUT_TYPE_HDMI:
    {
        strcpy(szInputType, "HDMI");
    }
        break;
    case MWCAP_VIDEO_INPUT_TYPE_SDI:
    {
        strcpy(szInputType, "SDI");
    }
        break;
    case MWCAP_VIDEO_INPUT_TYPE_VGA:
    {
        strcpy(szInputType, "VGA");
    }
        break;
    case MWCAP_VIDEO_INPUT_TYPE_COMPONENT:
    {
        strcpy(szInputType, "COMPONENT");
    }
        break;
    case MWCAP_VIDEO_INPUT_TYPE_CVBS:
    {
        strcpy(szInputType, "CVBS");
    }
        break;
    case MWCAP_VIDEO_INPUT_TYPE_YC:
    {
        strcpy(szInputType, "YC");
    }
        break;
    default:
        break;
    }

    return;

}

int main(void)
{
    MWCaptureInitInstance();
    MWRefreshDevice();

    BYTE nMajor = 0;
    BYTE nMinor = 0;
    WORD nBuild = 0;
    MWGetVersion(&nMajor, &nMinor, &nBuild);
    printf("MWCapture SDK for Linux V%d.%d.%d - video-set\n", nMajor, nMinor, nBuild);

    MW_RESULT mr;
    HCHANNEL hChannel;
    do
    {
        int nChannelCount = MWGetChannelCount();
        if (nChannelCount == 0)
        {
            printf("No Pro Capture device is found\n");
            break;
        }

        MWCAP_CHANNEL_INFO channelInfo;
        mr = MWGetChannelInfoByIndex(0, &channelInfo);
        if(mr != MW_SUCCEEDED)
        {
            printf("Error: Get channel info error\n");
            break;
        }

        hChannel = MWOpenChannel(channelInfo.byBoardIndex, channelInfo.byChannelIndex);
        if (hChannel == -1)
        {
            printf("Error: Open channel error\n");
            break;
        }

        printf("\nCapture channel info \n"
               "   %-13s : %d\n"
               "   %-13s : %d\n"
               "   %-13s : %s\n"
               "   %-13s : %s\n\n",
               "Board Index", channelInfo.byBoardIndex,
               "Channel Index", channelInfo.byChannelIndex,
               "Product Name", channelInfo.szProductName,
               "Serial Number", channelInfo.szBoardSerialNo);

        MWCAP_AUDIO_SIGNAL_STATUS status;
        mr = MWGetAudioSignalStatus(hChannel, &status);

        DWORD dwCurSource;
        mr = MWGetVideoInputSource(hChannel, &dwCurSource);
        if (mr != MW_SUCCEEDED)
        {
            printf("Error: Get video input source error\n");
            break;
        }
        BOOLEAN bScan = FALSE;
        char szInputType[32];
        mr = MWGetInputSourceScan(hChannel, &bScan);
        GetInPutTypestring(bScan, INPUT_TYPE(dwCurSource), szInputType);
        if (bScan)
            printf("AutoScan = YES\n");
        else
            printf("AutoScan = NO, Input type = %s\n",
                   szInputType
                   );



        DWORD dwVideoInputSources[VIDEO_INPUT_TYPE_NUM];
        DWORD dwVideoInputSourceCount = VIDEO_INPUT_TYPE_NUM;
        mr = MWGetVideoInputSourceArray(hChannel, dwVideoInputSources, &dwVideoInputSourceCount);
        if(mr != MW_SUCCEEDED)
        {
            printf("Error: Get video input sources arrray error\n");
            break;
        }
        
        PrintVideoInputSourceArray(dwVideoInputSources, dwVideoInputSourceCount, INPUT_TYPE(dwCurSource), bScan);

        printf("Selected video input source  'a' or (0 - %d)  ", dwVideoInputSourceCount - 1);
        fflush(stdout);

        char c = 0;
        do
        {
            c = mygetch();
            printf("\b%c", c);
            fflush(stdout);
        } while( (c < '0' || c > '0' + dwVideoInputSourceCount -1) && c != 'a');
        printf("\n\n");

        int nIndex = (int)(c - '0');
		if (c == 'a') {
            mr = MWSetInputSourceScan(hChannel, TRUE);
            //sleep(1);
		}
		else {
            mr = MWSetInputSourceScan(hChannel, FALSE);
            mr = MWSetVideoInputSource(hChannel, dwVideoInputSources[nIndex]);
            //printf("******* inputsource = 0x%x\n", dwVideoInputSources[nIndex]);
		}
        if (mr != MW_SUCCEEDED)
		{
            printf("Error: Set video intput souce error\n");
            break;
		}
        
        mr = MWGetVideoInputSource(hChannel, &dwCurSource);
        if (mr != MW_SUCCEEDED)
        {
            printf("Error: Get video input source error\n");
            break;
        }
        
        mr = MWGetInputSourceScan(hChannel, &bScan);
        //printf("AutoScan = %d, Input type = %d, source = 0x%x\n", bScan, INPUT_TYPE(dwCurSource), dwCurSource);

        GetInPutTypestring(bScan, INPUT_TYPE(dwCurSource), szInputType);

        //PrintVideoInputSourceArray(dwVideoInputSources, dwVideoInputSourceCount, INPUT_TYPE(dwCurSource), bScan);
        printf("Set video intput type to \"%s\" successfully\n", szInputType);

    } while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    MWCaptureExitInstance();

    return 0;
}
