////////////////////////////////////////////////////////////////////////////////
// CONFIDENTIAL and PROPRIETARY software of Magewell Electronics Co., Ltd.
// Copyright (c) 2011-2014 Magewell Electronics Co., Ltd. (Nanjing)
// All rights reserved.
// This copyright notice MUST be reproduced on all authorized copies.
////////////////////////////////////////////////////////////////////////////////
#include "videorenderfb.h"

#include <linux/vt.h>
#include <linux/kd.h>

static struct pixelformat pixelformattable[] = {
    {
        (char *)"RGB24",
        { 0, 8, 0 },
        { 8, 8, 0 },
        { 16, 8, 0 },
        { 0, 0, 0 },
        24,
        MWFOURCC_RGB24,
    },
    {
        (char *)"BGR24",
        { 16, 8, 0 },
        { 8, 8, 0 },
        { 0, 8, 0 },
        { 0, 0, 0 },
        24,
        MWFOURCC_BGR24,
    },
    {
        (char *)"ARGB",
        { 8, 8, 0 },
        { 16, 8, 0 },
        { 24, 8, 0 },
        { 0, 8, 0 },
        32,
        MWFOURCC_ARGB,
    },
    {
        (char *)"RGBA",
        { 0,8, 0 },
        { 8, 8, 0 },
        { 16, 8, 0 },
        { 24, 8, 0 },
        32,
        MWFOURCC_RGBA,
    },
    {
        (char *)"BGRA",
        { 16,8, 0 },
        { 8, 8, 0 },
        { 0, 8, 0 },
        { 24, 8, 0 },
        32,
        MWFOURCC_BGRA,
    },
    {
        (char *)"ABGR",
        { 24,8, 0 },
        { 16, 8, 0 },
        { 8, 8, 0 },
        { 0, 8, 0 },
        32,
        MWFOURCC_ABGR,
    }
};
#define PIXELFORMATNUM (sizeof(pixelformattable)/sizeof(struct pixelformat))

VideoRenderFB::VideoRenderFB() :
    m_ttyFd(-1),
    m_lastTv(-1),
    m_fbMemAddr(NULL)
{
    memset(&m_fbFix, 0, sizeof(m_fbFix));
    memset(&m_fbVar, 0, sizeof(m_fbVar));
}

VideoRenderFB::~VideoRenderFB()
{
    if (m_fbMemAddr != NULL)
        munmap(m_fbMemAddr, m_fbFix.smem_len);

    if(m_ttyFd > 0) {
        if (ioctl(m_ttyFd, KDSETMODE, KD_TEXT) < 0)
            perror("KDSETMODE");

        if (m_lastTv >= 0)
            if (ioctl(m_ttyFd, VT_ACTIVATE, m_lastTv))
                perror("VT_ACTIVATE");

        close(m_ttyFd);
    }
}

#include <signal.h>

static int tty_fd = -1;
static int lasttv = -1;
static bool g_switch_pending = false;

static void
LinuxSetSwitchMode(int fd, int mode);

static void
LinuxVTRequest(int sig)
{
    g_switch_pending = true;
    //printf("LinuxVTRequest\n");
}

static void
LinuxSetSwitchMode(int fd, int mode)
{
    struct sigaction act;
    struct vt_mode VT;

    if (ioctl(fd, VT_GETMODE, &VT) < 0) {
        perror("LinuxInit: VT_GETMODE failed\n");
    }

    if (mode == VT_PROCESS) {
        act.sa_handler = LinuxVTRequest;
        sigemptyset(&act.sa_mask);
        act.sa_flags = 0;
        sigaction(SIGUSR1, &act, 0);

        VT.mode = mode;
        VT.relsig = SIGUSR1;
        VT.acqsig = SIGUSR1;
    }
    else {
        act.sa_handler = SIG_IGN;
        sigemptyset(&act.sa_mask);
        act.sa_flags = 0;
        sigaction(SIGUSR1, &act, 0);

        VT.mode = mode;
        VT.relsig = 0;
        VT.acqsig = 0;
    }
    if (ioctl(fd, VT_SETMODE, &VT) < 0) {
        perror("LinuxInit: VT_SETMODE failed\n");
    }
}

bool VideoRenderFB::Init(const char *devname, int tnr)
{
    int ret;
    int fd, nr;
    char vtname[128];
    int con_fd;
    struct vt_stat vts;

    if (tnr != -2) {
        if (tnr >= 0) {
            nr = tnr;
        }else {
            fd = open("/dev/tty0", O_WRONLY);
            if (fd < 0) {
                mw_debug(MW_DEBUG_ERR, "Error opening concolse device %s: %s\n",
                         "/dev/tty0", strerror(errno));
                return false;
            }

            if (ioctl(fd, VT_OPENQRY, &nr) < 0) {
                mw_debug(MW_DEBUG_ERR, "ioctl VT_OPENQRY error: %s\n", strerror(errno));
                return false;
            }
            close(fd);
        }

        sprintf(vtname, "%s%d", "/dev/tty", nr);

        con_fd = open(vtname, O_RDWR | O_NDELAY);
        if (con_fd < 0) {
            mw_debug(MW_DEBUG_ERR, "Error open tty %s: %s\n",
                     vtname, strerror(errno));
            return false;
        }

        ///*
        LinuxSetSwitchMode(con_fd, VT_AUTO);
        if (ioctl(con_fd, VT_GETSTATE, &vts) == 0)
            m_lastTv = vts.v_active;

        if (ioctl(con_fd, VT_ACTIVATE, nr) < 0) {
            perror("VT_ACTIVATE");
            close(con_fd);
            return false;
        }

        if (ioctl(con_fd, VT_WAITACTIVE, nr) < 0) {
            perror("VT_WAITACTIVE");
            close(con_fd);
            return false;
        }
        //*/

        LinuxSetSwitchMode(con_fd, VT_PROCESS);
        ///*
        if (ioctl(con_fd, KDSETMODE, KD_GRAPHICS) < 0) {
            perror("KDSETMODE");
            close(con_fd);
            return false;
        }
        //*/
        tty_fd = con_fd;
        lasttv = m_lastTv;

        m_ttyFd = con_fd;
    }

    m_fbHandler = open(devname, O_RDWR);
    if (m_fbHandler < 0){
        mw_debug(MW_DEBUG_ERR, "Error opening framebuffer device: %s\n", devname);
        return false;
    }
    ret = ioctl(m_fbHandler, FBIOGET_FSCREENINFO, &m_fbFix);
    if (ret < 0){
        mw_debug(MW_DEBUG_ERR, "Error getting fix screeninfo\n");
        return false;
    }
    ret = ioctl(m_fbHandler, FBIOGET_VSCREENINFO, &m_fbVar);
    if (ret < 0){
        mw_debug(MW_DEBUG_ERR, "Error getting var screeninfo\n");
        return false;
    }
    /*
    ret = ioctl(m_fbHandler, FBIOPUT_VSCREENINFO, &m_fbVar);
    if (ret < 0){
        mw_debug(MW_DEBUG_ERR, "Error setting mode\n");
        return false;
    }
    */

    m_fbMemAddr = (unsigned char *)mmap(0, m_fbFix.smem_len, PROT_READ|PROT_WRITE, MAP_SHARED, m_fbHandler, 0);
    if (m_fbMemAddr == MAP_FAILED){
        mw_debug(MW_DEBUG_ERR, "Error mmap'ing framebuffer device\n");
        return false;
    }

    /*
    mw_debug(MW_DEBUG_INFO, "m_fbVar.bits_per_pixel=%d\n",
             m_fbVar.bits_per_pixel);
    mw_debug(MW_DEBUG_INFO, "m_fbVar.red: %d %d\n",
             m_fbVar.red.offset, m_fbVar.red.length);
    mw_debug(MW_DEBUG_INFO, "var_check.green: %d %d\n",
             m_fbVar.green.offset, m_fbVar.green.length);
    mw_debug(MW_DEBUG_INFO, "m_fbVar.blue: %d %d\n",
             m_fbVar.blue.offset, m_fbVar.blue.length);
    mw_debug(MW_DEBUG_INFO, "m_fbVar.transp: %d %d\n",
             m_fbVar.transp.offset, m_fbVar.transp.length);
             */
    for (int i = 0; i < (int)PIXELFORMATNUM; i++) {
        /*
        mw_debug(MW_DEBUG_INFO, "pixelformattable[%d].red: %d %d\n",
                 i, pixelformattable[i].red.offset, pixelformattable[i].red.length);
        mw_debug(MW_DEBUG_INFO, "pixelformattable[%d].green: %d %d\n",
                 i, pixelformattable[i].green.offset, pixelformattable[i].green.length);
        mw_debug(MW_DEBUG_INFO, "pixelformattable[%d].blue: %d %d\n",
                 i, pixelformattable[i].blue.offset, pixelformattable[i].blue.length);
        mw_debug(MW_DEBUG_INFO, "pixelformattable[%d].transp: %d %d\n",
                 i, pixelformattable[i].transp.offset, pixelformattable[i].transp.length);
                 */

        /*
        if (setmode(&pixelformattable[i], m_fbVar.xres, m_fbVar.yres) == 0) {
            struct pixelformat *pix_temp = &pixelformattable[i];
            m_arSupportFourcc.append(pix_temp);
        }
        */

        struct pixelformat *pixf = &pixelformattable[i];
        if (m_fbVar.red.offset == pixf->red.offset
                && m_fbVar.red.length == pixf->red.length
                && m_fbVar.green.offset == pixf->green.offset
                && m_fbVar.green.length == pixf->green.length
                && m_fbVar.blue.offset == pixf->blue.offset
                && m_fbVar.blue.length == pixf->blue.length
                && (int)m_fbVar.bits_per_pixel == pixf->bpp
                ) {
            m_arSupportFourcc.append(pixf);
        }
    }

    return true;
}

uint32_t *VideoRenderFB::GetSupportedFourcc(int &nCount)
{
    uint32_t *newData = NULL;
    nCount = 0;

    if (m_arSupportFourcc.size() <= 0)
        return NULL;

    newData = new uint32_t[m_arSupportFourcc.size()];
    if (newData == NULL)
        return NULL;

    nCount = m_arSupportFourcc.size();
    for (int i = 0; i < nCount; i++) {
        newData[i] = m_arSupportFourcc[i]->mwfourcc;
    }
    return newData;
}

bool VideoRenderFB::SetFrameFormat(int nWidth, int nHeight, uint32_t dwColorSpace, bool bTry)
{
    //int ret;

    if (nWidth != (int)m_fbVar.xres
            || nHeight != (int)m_fbVar.yres)
        return false;

    int i = 0;
    for (i = 0; i < (int)m_arSupportFourcc.size(); i++) {
        if (m_arSupportFourcc[i]->mwfourcc == dwColorSpace)
            break;
    }
    if (i >= (int)m_arSupportFourcc.size())
        return false;

    /*
    if (setmode(m_arSupportFourcc[i], nWidth, nHeight) < 0)
        return false;

    ret = ioctl(m_fbHandler, FBIOGET_VSCREENINFO, &m_fbVar);
    if (ret < 0){
        mw_debug(MW_DEBUG_ERR, "Error getting var screeninfo\n");
        return false;
    }

    mw_debug(MW_DEBUG_INFO, "m_fbVar.bits_per_pixel=%d\n", m_fbVar.bits_per_pixel);
    */

    if (!bTry) {
        m_nWidth = nWidth;
        m_nHeight = nHeight;
        m_dwColorSpace = dwColorSpace;
        m_BufferSize = FOURCC_CalcImageSize(
                    dwColorSpace, nWidth, nHeight,
                    FOURCC_CalcMinStride(dwColorSpace, nWidth, 4));
    }
    return true;
}

VideoRenderImageBuf *VideoRenderFB::AllocImageBuffer(VideoRenderImageBufType type) {
    if (type != VIDEO_RENDER_IMAGEBUF_VIRTUAL)
        return NULL;

    if (m_BufferSize == 0) {
        mw_debug(MW_DEBUG_ERR, "Set frame format first!\n");
        return NULL;
    }

    VideoRenderImageBuf *pImageBuf = new VideoRenderImageBuf;
    if (pImageBuf == NULL)
        return NULL;

    pImageBuf->buf = malloc(m_BufferSize);

    if (pImageBuf->buf == NULL) {
        delete pImageBuf;
        return NULL;
    }

    pImageBuf->type = VIDEO_RENDER_IMAGEBUF_VIRTUAL;
    pImageBuf->size = m_BufferSize;
    return pImageBuf;
}
void VideoRenderFB::FreeImageBuffer(VideoRenderImageBuf *pImageBuf) {
    if (pImageBuf->buf != NULL) {
        free(pImageBuf->buf);
        pImageBuf->buf = NULL;
    }

    delete pImageBuf;
}

static char _get_user_ch()
{
    fd_set rfds;
    struct timeval tv;
    int retval;
    char ch[16];

    /* Watch stdin (fd 0) to see when it has input. */
    FD_ZERO(&rfds);
    FD_SET(0, &rfds);

    /* Wait up to five seconds. */
    tv.tv_sec = 0;
    tv.tv_usec = 100;

    retval = select(1, &rfds, NULL, NULL, &tv);
    /* Don't rely on the value of tv now! */

    if (retval) {
        fgets(ch, 16, stdin);
        if(ch[1] != '\n' || ch[0] == '\n')
            ch[0] = '\n';
        return ch[0];
    }

    return 0;
}

void VideoRenderFB::RunLoop(ExitCleanCallback exit_call, void *data) {
    char ch;

    printf("Press 'q' to exit\n");

    while (1) {
        ch = _get_user_ch();

        if (ch == 'q') {
            if (exit_call != NULL)
                exit_call(data);
            break;
        }
    }
}

int VideoRenderFB::drawimage(unsigned char *buffer, size_t size)
{
    unsigned int y;
    int stride = FOURCC_CalcMinStride(m_dwColorSpace,
                                      m_nWidth, 4);

    // TODO
#if 0
    if (g_switch_pending) {
        g_switch_pending = false;
    }
#endif

    for(y = 0; y < m_fbVar.yres; y++) {
        if (stride * (y+1) > size) {
            printf("break\n");
            break;
        }
        long fb_offset = (m_fbVar.xoffset) * (m_fbVar.bits_per_pixel/8)
            + (y + m_fbVar.yoffset) * m_fbFix.line_length;
        long img_offset = y * stride;

        for (unsigned int x = 0; x < m_fbVar.xres; x++) {
            unsigned char tr, tg, tb, ta;

            ta = 0;
            switch (m_dwColorSpace) {
            case MWFOURCC_RGB24:
                tr = *(buffer + img_offset + 0);
                tg = *(buffer + img_offset + 1);
                tb = *(buffer + img_offset + 2);
                break;
            case MWFOURCC_RGBA:
                tr = *(buffer + img_offset + 0);
                tg = *(buffer + img_offset + 1);
                tb = *(buffer + img_offset + 2);
                ta = *(buffer + img_offset + 3);
                break;
            case MWFOURCC_ARGB:
                tr = *(buffer + img_offset + 1);
                tg = *(buffer + img_offset + 2);
                tb = *(buffer + img_offset + 3);
                ta = *(buffer + img_offset + 0);
                break;
            case MWFOURCC_BGR24:
                tr = *(buffer + img_offset + 2);
                tg = *(buffer + img_offset + 1);
                tb = *(buffer + img_offset + 0);
                break;
            case MWFOURCC_BGRA:
                tr = *(buffer + img_offset + 2);
                tg = *(buffer + img_offset + 1);
                tb = *(buffer + img_offset + 0);
                ta = *(buffer + img_offset + 3);
                break;
            case MWFOURCC_ABGR:
                tr = *(buffer + img_offset + 3);
                tg = *(buffer + img_offset + 2);
                tb = *(buffer + img_offset + 1);
                ta = *(buffer + img_offset + 0);
                break;
            }

            unsigned int tvalue;
            switch (m_fbVar.bits_per_pixel) {
            case 24:
                /*
                tvalue = (tr << m_fbVar.red.offset) |
                        (tg << m_fbVar.green.offset) |
                        (tb << m_fbVar.blue.offset);
                *(unsigned short*)(m_fbMemAddr + fb_offset) = tvalue & 0xffff;
                *(m_fbMemAddr + fb_offset + 2) = (tvalue >> 16) & 0xff;
                * */
                *(m_fbMemAddr + fb_offset + (m_fbVar.red.offset/8)) = tr;
                *(m_fbMemAddr + fb_offset + (m_fbVar.green.offset/8)) = tg;
                *(m_fbMemAddr + fb_offset + (m_fbVar.blue.offset/8)) = tb;
                break;
            case 32:
                tvalue = (tr << m_fbVar.red.offset) |
                        (tg << m_fbVar.green.offset) |
                        (tb << m_fbVar.blue.offset) |
                        (ta << m_fbVar.transp.offset);
                *(unsigned int*)(m_fbMemAddr + fb_offset) = tvalue;
                break;
            }

            fb_offset += m_fbVar.bits_per_pixel / 8;
            img_offset += FOURCC_GetBpp(m_dwColorSpace) / 8;
        }
    }

    return 0;
}

void VideoRenderFB::DisplayFrame(VideoRenderImageBuf *imageBuf) {
    if (m_fbHandler > 0 && m_fbMemAddr != NULL) {
        size_t _size = imageBuf->size < m_fbFix.smem_len ?
                    imageBuf->size : m_fbFix.smem_len;
        memcpy(m_fbMemAddr, imageBuf->buf, _size);
        //drawimage((unsigned char *)imageBuf->buf, _size);
    }
}
void VideoRenderFB::DisplayFrame(void *buffer, size_t size) {
    if (m_fbHandler > 0 && m_fbMemAddr != NULL) {
        size_t _size = size < m_fbFix.smem_len ?
                    size : m_fbFix.smem_len;
        //drawimage((unsigned char *)buffer, _size);
        memcpy(m_fbMemAddr, buffer, _size);
    }
}

void VideoRenderFB::DisplayFps(unsigned long long frameCount, double fps) {
    printf("\r\033[K"); // clear line
    printf("mwcap-v4l2  [total frame]: %lld [average fps]: %.2f", frameCount, fps);
    fflush(stdout);
}

//private
int VideoRenderFB::setmode(const struct pixelformat *pixf, int width, int height)
{
    struct fb_var_screeninfo var_old;
    struct fb_var_screeninfo var_new;
    struct fb_var_screeninfo var_check;
    int stat;

    stat = ioctl(m_fbHandler, FBIOGET_VSCREENINFO, &var_old);
    if (stat < 0)
        return stat;

    var_new = var_old;

    var_new.xres= width;
    var_new.xres_virtual = width;
    var_new.yres= height;
    var_new.yres_virtual = height;

    var_new.bits_per_pixel = pixf->bpp;
    var_new.red = pixf->red;
    var_new.green = pixf->green;
    var_new.blue = pixf->blue;
    var_new.transp = pixf->transp;

    /*
    mw_debug(MW_DEBUG_INFO, "==== start\n");
    mw_debug(MW_DEBUG_INFO, "var_new.red: %d %d\n",
             var_new.red.offset, var_new.red.length);
    mw_debug(MW_DEBUG_INFO, "var_new.green: %d %d\n",
             var_new.green.offset, var_new.green.length);
    mw_debug(MW_DEBUG_INFO, "var_new.blue: %d %d\n",
             var_new.blue.offset, var_new.blue.length);
    mw_debug(MW_DEBUG_INFO, "var_new.bits_per_pixel=%d\n",
             var_new.bits_per_pixel);
    */

    stat = ioctl(m_fbHandler, FBIOPUT_VSCREENINFO, &var_new);
    if (stat < 0)
        return stat;

    //mw_debug(MW_DEBUG_INFO, "====\n");
    stat = ioctl(m_fbHandler, FBIOGET_VSCREENINFO, &var_check);
    if (stat < 0)
        return stat;
    /*
    mw_debug(MW_DEBUG_INFO, "var_check.bits_per_pixel=%d\n",
             var_check.bits_per_pixel);
    mw_debug(MW_DEBUG_INFO, "var_check.red: %d %d\n",
             var_check.red.offset, var_check.red.length);
    mw_debug(MW_DEBUG_INFO, "var_check.green: %d %d\n",
             var_check.green.offset, var_check.green.length);
    mw_debug(MW_DEBUG_INFO, "var_check.blue: %d %d\n",
             var_check.blue.offset, var_check.blue.length);

    mw_debug(MW_DEBUG_INFO, "==== end\n");
    */

    if ((int)var_check.xres == width
            && (int)var_check.yres == height
            && var_check.red.offset == pixf->red.offset
            && var_check.red.length == pixf->red.length
            && var_check.green.offset == pixf->green.offset
            && var_check.green.length == pixf->green.length
            && var_check.blue.offset == pixf->blue.offset
            && var_check.blue.length == pixf->blue.length
            && var_check.transp.offset == pixf->transp.offset
            && var_check.transp.length == pixf->transp.length
            && (int)var_check.bits_per_pixel == pixf->bpp
            )
        return 0;

    stat = ioctl(m_fbHandler, FBIOPUT_VSCREENINFO, &var_old);
    if (stat < 0)
        return stat;

    return -1;
}
