////////////////////////////////////////////////////////////////////////////////
// CONFIDENTIAL and PROPRIETARY software of Magewell Electronics Co., Ltd.
// Copyright (c) 2011-2014 Magewell Electronics Co., Ltd. (Nanjing)
// All rights reserved.
// This copyright notice MUST be reproduced on all authorized copies.
////////////////////////////////////////////////////////////////////////////////
#ifndef VIDEORENDERNULL_H
#define VIDEORENDERNULL_H

#include "videorenderbase.h"

#include <stdlib.h>

class VideoRenderNull : public VideoRenderBase
{
public:
    VideoRenderNull()
    {
    }

    ~VideoRenderNull()
    {
        printf("\n");
    }

    bool Init() { return true; }

    char *GetName() { return (char *)"null"; }

    virtual uint32_t *GetSupportedFourcc(int &nCount) {
        nCount = 0;
        return NULL;
    }

    virtual bool SetFrameFormat(int nWidth, int nHeight, uint32_t dwColorSpace, bool bTry = false) {
        if (!bTry) {
            m_nWidth = nWidth;
            m_nHeight = nHeight;
            m_dwColorSpace = dwColorSpace;
            m_BufferSize = FOURCC_CalcImageSize(
                        dwColorSpace, nWidth, nHeight,
                        FOURCC_CalcMinStride(dwColorSpace, nWidth, 4));
        }
        return true;
    }

    virtual bool IsSupportImageBufType(VideoRenderImageBufType type) {
        if (type == VIDEO_RENDER_IMAGEBUF_VIRTUAL)
            return true;

        return false;
    }
    virtual VideoRenderImageBuf *AllocImageBuffer(VideoRenderImageBufType type) {
        if (type != VIDEO_RENDER_IMAGEBUF_VIRTUAL)
            return NULL;

        if (m_BufferSize == 0) {
            mw_debug(MW_DEBUG_ERR, "Set frame format first!\n");
            return NULL;
        }

        VideoRenderImageBuf *pImageBuf = new VideoRenderImageBuf;
        if (pImageBuf == NULL)
            return NULL;

        pImageBuf->buf = malloc(m_BufferSize);

        if (pImageBuf->buf == NULL) {
            delete pImageBuf;
            return NULL;
        }

        pImageBuf->type = VIDEO_RENDER_IMAGEBUF_VIRTUAL;
        pImageBuf->size = m_BufferSize;
        return pImageBuf;
    }
    virtual void FreeImageBuffer(VideoRenderImageBuf *pImageBuf) {
        if (pImageBuf->buf != NULL) {
            free(pImageBuf->buf);
            pImageBuf->buf = NULL;
        }

        delete pImageBuf;
    }

    static char _get_user_ch()
    {
        fd_set rfds;
        struct timeval tv;
        int retval;
        char ch[16];

        /* Watch stdin (fd 0) to see when it has input. */
        FD_ZERO(&rfds);
        FD_SET(0, &rfds);

        /* Wait up to five seconds. */
        tv.tv_sec = 0;
        tv.tv_usec = 100;

        retval = select(1, &rfds, NULL, NULL, &tv);
        /* Don't rely on the value of tv now! */

        if (retval) {
            fgets(ch, 16, stdin);
            if(ch[1] != '\n' || ch[0] == '\n')
                ch[0] = '\n';
            return ch[0];
        }

        return 0;
    }

    virtual void RunLoop(ExitCleanCallback exit_call, void *data) {
        char ch;

        printf("Press 'q' to exit\n");

        while (1) {
            ch = _get_user_ch();

            if (ch == 'q') {
                if (exit_call != NULL)
                    exit_call(data);
                break;
            }
        }
    }

    virtual void DisplayFrame(VideoRenderImageBuf *imageBuf) {}
    virtual void DisplayFrame(void *buffer, size_t size) {}

    virtual void DisplayFps(unsigned long long frameCount, double fps) {
        printf("\r\033[K"); // clear line
        printf("mwcap-v4l2  [total frame]: %lld [average fps]: %.2f", frameCount, fps);
        fflush(stdout);
    }
};

#endif // VIDEORENDERNULL_H
