#include "cdlgaddresolution.h"

CDlgAddResolution::CDlgAddResolution(int nIndex, QDialog *parent) : QDialog(parent)
{
    m_nIndex = nIndex;
    QLabel* lblWidth = new QLabel("Width:");
    QLabel* lblHeight = new QLabel("Height:");

    m_editWidth = new QLineEdit;
    m_editHeight = new QLineEdit;

    QRegExp regx("[1-9][0-9]+$");
    QValidator *validator = new QRegExpValidator(regx, m_editWidth);
    m_editWidth->setValidator(validator);
    QValidator *validatorHeight = new QRegExpValidator(regx, m_editHeight);
    m_editHeight->setValidator(validatorHeight);

    QGridLayout* glayout = new QGridLayout;
    glayout->addWidget(lblWidth, 0, 0, 1, 1);
    glayout->addWidget(m_editWidth, 0, 1, 1, 2);
    glayout->addWidget(lblHeight, 1, 0, 1, 1);
    glayout->addWidget(m_editHeight, 1, 1, 1, 2);

    m_btnOK = new QPushButton("OK");
    m_btnCancel = new QPushButton("Cancel");
    QHBoxLayout* hlayoutButton = new QHBoxLayout;
    hlayoutButton->addStretch();
    hlayoutButton->addWidget(m_btnOK);
    hlayoutButton->addWidget(m_btnCancel);

    QVBoxLayout* vlayoutMain = new QVBoxLayout;
    vlayoutMain->addLayout(glayout);
    vlayoutMain->addLayout(hlayoutButton);
    this->setLayout(vlayoutMain);

    UpdateItemValue(m_nIndex);
    connect(m_btnOK, SIGNAL(clicked()), this, SLOT(OnBtnOKClicked()));
    connect(m_btnCancel, SIGNAL(clicked()), this, SLOT(reject()));
}

void CDlgAddResolution::OnBtnOKClicked()
{
    QString strWidth = m_editWidth->text();
    QString strHeight = m_editHeight->text();
    if (strWidth.isEmpty()) {
        QMessageBox::about(this, "Error", "Enter a number between 128 and 4096");
        m_editWidth->setFocus();
        return;
    }
    if (strHeight.isEmpty()) {
        QMessageBox::about(this, "Error", "Enter a number between 128 and 4096");
        m_editHeight->setFocus();
        return;
    }

    int m_nWidth = strWidth.toInt();
    int m_nHeight = strHeight.toInt();

    if (m_nWidth < 128 || m_nWidth > 4096 || m_nHeight < 128 || m_nHeight > 4096) {
        QMessageBox::about(this, "Error", "Enter a number between 128 and 4096");
        return ;
    }

    DWORD dwResolutionCount = 0;
    MW_RESULT nRet = MWGetCustomVideoResolutionsCount(g_capture.GetVideoHandle(), &dwResolutionCount);
    DWORD dwBufferSize = 32;
    MWCAP_SIZE szVideoResolution[32];
    nRet = MWGetCustomVideoResolutionsArray(g_capture.GetVideoHandle(), szVideoResolution, &dwBufferSize);

    if (m_nIndex == -1) { //add
        szVideoResolution[dwResolutionCount].cx = m_nWidth;
        szVideoResolution[dwResolutionCount].cy = m_nHeight;
        ++ dwResolutionCount;
    }
    else if(m_nIndex == -2)
    {
        szVideoResolution[0].cx = m_nWidth;
        szVideoResolution[0].cy = m_nHeight;
        dwResolutionCount = 1;
    }
    else { // modify
        szVideoResolution[m_nIndex].cx = m_nWidth;
        szVideoResolution[m_nIndex].cy = m_nHeight;
    }

    nRet = MWSetCustomVideoResolutionsArray(g_capture.GetVideoHandle(), szVideoResolution, dwResolutionCount);

    close();
}

void CDlgAddResolution::OnBtnCancelClicked()
{

}

void CDlgAddResolution::UpdateItemValue( int nIndex )
{
    if (nIndex != -1 && nIndex != -2) {
        DWORD dwResolutionCount = 0;
        MW_RESULT xr = MWGetCustomVideoResolutionsCount(g_capture.GetVideoHandle(), &dwResolutionCount);
        DWORD dwBufferSize = 32;
        MWCAP_SIZE szVideoResolution[32];
        xr = MWGetCustomVideoResolutionsArray(g_capture.GetVideoHandle(), szVideoResolution, &dwBufferSize);

        MWCAP_SIZE szSel = szVideoResolution[nIndex];

        m_editWidth->setText(QString("%1").arg(szSel.cx));
        m_editHeight->setText(QString("%1").arg(szSel.cy));
    }
    else {
        m_editWidth->setText("");
        m_editHeight->setText("");
    }
}
