#include "cdlginfo.h"
CDlgInfo::CDlgInfo(QWidget *parent) : QWidget(parent)
{
    m_tableInfo = new QTableWidget(this);
    QStringList lstHorHeadName;
    lstHorHeadName << "Name" << "Value";

    m_tableInfo->setColumnCount(2);
    m_tableInfo->setHorizontalHeaderLabels(lstHorHeadName);
    m_tableInfo->setColumnWidth(0, 200);
    m_tableInfo->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_tableInfo->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_tableInfo->horizontalHeader()->setStretchLastSection(true);

    QHBoxLayout *playMain = new QHBoxLayout;
    playMain->addWidget(m_tableInfo);
    this->setLayout(playMain);

    UpdateInfoTable();
    QApplication::connect(&m_tempTimer, SIGNAL(timeout()), this, SLOT(onTempTimerTimeOut()));
    m_tempTimer.start(500);
}

inline void CDlgInfo::AddItem(QList<TABLE_ITEM> &lstTableitem, QString key, QString value)
{
    TABLE_ITEM item = {key, value};
    lstTableitem.append(item);
}

void CDlgInfo::UpdateInfoTable()
{
    //channel info
    MWCAP_CHANNEL_INFO channelInfo;
    QList<TABLE_ITEM>   lstTableitem;
    MWGetChannelInfo(g_capture.GetVideoHandle(), &channelInfo);
    AddItem(lstTableitem, "Family name", QString("%1").arg(channelInfo.szFamilyName));
    AddItem(lstTableitem, "Product name", QString("%1").arg(channelInfo.szProductName));
    AddItem(lstTableitem, "Firmware name", QString("%1").arg(channelInfo.szFirmwareName));
    AddItem(lstTableitem, "Serial number", QString("%1").arg(channelInfo.szBoardSerialNo));
    AddItem(lstTableitem, "Hardware version", QString("%1").arg(channelInfo.chHardwareVersion));
    AddItem(lstTableitem, "Firmware version", QString("%1.%2").arg(channelInfo.dwFirmwareVersion >> 16).arg(channelInfo.dwFirmwareVersion & 0xffff));
    AddItem(lstTableitem, "Driver version", QString("%1.%2.%3").arg(channelInfo.dwDriverVersion >> 24).arg((channelInfo.dwDriverVersion >> 16) & 0x00ff).arg(channelInfo.dwDriverVersion & 0xffff));
    AddItem(lstTableitem, "Border index", QString("%1").arg(channelInfo.byBoardIndex));
    AddItem(lstTableitem, "Channel index", QString("%1").arg(channelInfo.byChannelIndex));

    //pic-e info
    MWCAP_PRO_CAPTURE_INFO captureInfo;
    MWGetFamilyInfo(g_capture.GetVideoHandle(), &captureInfo, sizeof(captureInfo));
    AddItem(lstTableitem, "PCIe address", QString("Bus %1, Device %2").arg(captureInfo.byPCIBusID).arg(captureInfo.byPCIDevID));
    AddItem(lstTableitem, "PCIe speed", QString("Gen %1").arg(captureInfo.byLinkType));
    AddItem(lstTableitem, "PCIe width", QString("x%1").arg(captureInfo.byLinkWidth));
    AddItem(lstTableitem, "PCIe max payload size", QString("%1 Bytes").arg(captureInfo.wMaxPayloadSize));
    AddItem(lstTableitem, "PCIe max read request size", QString("%1 Bytes").arg(captureInfo.wMaxReadRequestSize));
    AddItem(lstTableitem, "Total memory size", QString("%1 Bytes").arg(captureInfo.cbTotalMemorySize));
    AddItem(lstTableitem, "Free memory size", QString("%1 Bytes").arg(captureInfo.cbFreeMemorySize));

    //video caps
    MWCAP_VIDEO_CAPS videoCaps;
    MWGetVideoCaps(g_capture.GetVideoHandle(), &videoCaps);
    AddItem(lstTableitem, "Max input dimension", QString("%1x%2 Pixels").arg(videoCaps.wMaxInputWidth).arg(videoCaps.wMaxInputHeight));
    AddItem(lstTableitem, "Max output dimension", QString("%1x%2 Pixels").arg(videoCaps.wMaxOutputWidth).arg(videoCaps.wMaxOutputHeight));

    //temperature
    DWORD dwTemperatureDegC_X10 = 0;
    MWGetTemperature(g_capture.GetVideoHandle(), &dwTemperatureDegC_X10);
    AddItem(lstTableitem, "Core temperature", QString("%1.%2 deg C").arg(dwTemperatureDegC_X10 / 10).arg(dwTemperatureDegC_X10 % 10));

    m_tableInfo->clearContents();
    m_tableInfo->setRowCount(lstTableitem.size());
    for(int i = 0; i < lstTableitem.size(); i++)
    {
        m_tableInfo->setItem(i, 0, new QTableWidgetItem(lstTableitem.at(i).key));
        m_tableInfo->setItem(i, 1, new QTableWidgetItem(lstTableitem.at(i).value));
    }

}

//slots
void CDlgInfo::onTempTimerTimeOut()
{
    DWORD dwTemperatureDegC_X10 = 0;
    MWGetTemperature(g_capture.GetVideoHandle(), &dwTemperatureDegC_X10);
    int nRowCount = m_tableInfo->rowCount();
    m_tableInfo->setItem((nRowCount - 1), 1, new QTableWidgetItem(QString("%1.%2 deg C").arg(dwTemperatureDegC_X10 / 10).arg(dwTemperatureDegC_X10 % 10)));
}
