#include "cdlgosd.h"

CDlgOSD::CDlgOSD(QWidget *parent) : QWidget(parent)
{
    m_strImagePath = "";

    m_lblOSDImage = new QLabel(this);
    m_lblOSDImage->setFrameShape(QFrame::Panel);
    m_editImagePath = new QLineEdit(this);
    m_editImagePath->setEnabled(false);
    m_editImagePath->setLayoutDirection(Qt::RightToLeft);
//    m_editImagePath->setAlignment(Qt::AlignLeft);
    m_btnSelectImage = new QPushButton("...", this);
    m_ckbEnableOSD = new QCheckBox("Enable OSD", this);
    m_btnLoadPresent = new QPushButton("Load present", this);
    m_btnSavePresent = new QPushButton("Save present", this);

    m_ckbEnableOSD->setFocusPolicy(Qt::NoFocus);
    m_ckbEnableOSD->setEnabled(false);
//    m_ckbEnableOSD->setStyleSheet("background-color:transparent");

    QGridLayout *playoutBottom = new QGridLayout;
    playoutBottom->addWidget(m_editImagePath, 0, 0, 1, 3);
    playoutBottom->addWidget(m_btnSelectImage, 0, 3);
    playoutBottom->addWidget(m_ckbEnableOSD, 1, 0);
    playoutBottom->addWidget(m_btnLoadPresent, 1, 2);
    playoutBottom->addWidget(m_btnSavePresent, 1, 3);

    playoutBottom->setColumnStretch(0, 2);
    playoutBottom->setColumnStretch(1,2);
    playoutBottom->setColumnStretch(2,1);
    playoutBottom->setColumnStretch(3,1);

    QLabel *plblPath = new QLabel("OSD file path(PNG RGBA file):", this);
    QVBoxLayout *pVlayoutMain = new QVBoxLayout;
    pVlayoutMain->addWidget(m_lblOSDImage, 3);
    pVlayoutMain->addWidget(plblPath);
    pVlayoutMain->addLayout(playoutBottom);
    pVlayoutMain->addStretch(1);

    this->setLayout(pVlayoutMain);
    UpdateOSD();
    QApplication::connect(m_btnSelectImage, SIGNAL(clicked(bool)), this, SLOT(onSelectImageClicked(bool)));
    QApplication::connect(m_btnSavePresent, SIGNAL(clicked(bool)), this, SLOT(onSavePresentClicked(bool)));
    QApplication::connect(m_btnLoadPresent, SIGNAL(clicked(bool)), this, SLOT(onLoadPresentClicked(bool)));
    QApplication::connect(m_ckbEnableOSD, SIGNAL(stateChanged(int)), this, SLOT(onEnableOSDChecked(int)));

    m_shFile = new CGenerateSHFile(this);
}

void CDlgOSD::UpdateOSD()
{
    onLoadPresentClicked(true);
}

//slots
void CDlgOSD::onSelectImageClicked(bool bFlag)
{
    QString strPath = QFileDialog::getOpenFileName(this, tr("Select PNG image"), m_strImagePath, "Image Files(*.png)");
    if (strPath == "")
        return ;

    m_ckbEnableOSD->setEnabled(true);
    m_strImagePath = strPath;
    if(UploadImage(strPath) == true)
    {
        m_lblOSDImage->clear();
        m_editImagePath->setText(strPath);
        m_editImagePath->setCursorPosition(0);
        m_lblOSDImage->setStyleSheet(QString("border-image:url(%1)").arg(strPath));
    }
}

void CDlgOSD::onLoadPresentClicked(bool bFlag)
{
    MWCAP_VIDEO_OSD_SETTINGS videoOSDSettings;
    MWGetVideoOSDSettings(g_capture.GetVideoHandle(), &videoOSDSettings);
    QString strPath = QString("%1").arg(videoOSDSettings.szPNGFilePath);
    m_editImagePath->setText(strPath);
    m_editImagePath->setCursorPosition(0);
    m_ckbEnableOSD->setChecked(videoOSDSettings.bEnable);
    if (!strPath.isEmpty()) {
        m_lblOSDImage->clear();
        m_lblOSDImage->setStyleSheet(QString("border-image:url(%1)").arg(strPath));
        UploadImage(strPath);

        if (videoOSDSettings.bEnable) {
            g_capture.SetEnableOSD(videoOSDSettings.bEnable);
        }
    }else
    {
        m_lblOSDImage->clear();
        m_lblOSDImage->setStyleSheet("QLabel{border:1px solid rgb(200, 200, 200)}");
        m_ckbEnableOSD->setEnabled(false);
    }

    g_capture.SetEnableOSD(videoOSDSettings.bEnable);
}

void CDlgOSD::onSavePresentClicked(bool bFlag)
{
    MWCAP_VIDEO_OSD_SETTINGS videoOSDSettings = {0};
    videoOSDSettings.bEnable = m_ckbEnableOSD->isChecked();

    QString strPath = m_editImagePath->text();
    if (!strPath.isEmpty()) {
        QByteArray ba = strPath.toLocal8Bit();
        char* pszPath = ba.data();
        strcpy(videoOSDSettings.szPNGFilePath, pszPath);
    }

    MWSetVideoOSDSettings(g_capture.GetVideoHandle(), videoOSDSettings);
}

void CDlgOSD::onEnableOSDChecked(int nStatus)
{
    bool bCheck = nStatus == Qt::Checked ? true : false;
    if (bCheck && !m_editImagePath->text().isEmpty() && QFile::exists(m_editImagePath->text())) {
        bCheck = true;
    }
    else {
        bCheck = false;
    }
    g_capture.SetEnableOSD(bCheck);

    //v4l2
   // g_capture.SetOSDImage(m_editImagePath->text().toLatin1().data(), bCheck);
}

//private
bool CDlgOSD::UploadImage(QString &pngpath)
{
    if (pngpath.isEmpty() == true)
        return false;

    //private stream
    QImage image;
    bool bPNG = image.load(pngpath, "png");
    if(bPNG == false)
    {
        QMessageBox::warning(this, "OSD", "Load OSD file failed ! The OSD format must be PNG (176x120-2048x2048).");
        return false;
    }
    uchar* pngimage = image.bits();
    WORD pngwidth = image.width();
    WORD pngheight = image.height();

    if((pngwidth < 176 || pngwidth > 2048) || (pngheight < 120 || pngheight > 2048))
    {
        QMessageBox::warning(this, "OSD", "Load OSD file failed ! The OSD format must be PNG (176x120-2048x2048).");
        return false;
    }

    g_capture.UploadOSDImage((unsigned char *)pngimage, pngwidth, pngheight);

    //v4l2
   // g_capture.SetOSDImage(pngpath, m_ckbEnableOSD->isChecked());

    return true;
}
