#ifndef CDLGTIMING_H
#define CDLGTIMING_H

#include <QWidget>
#include "cvideochannel.h"
#include "publicheaders.h"
#include "ctablewidget.h"
#include "cdlgaddresolution.h"
#include "isignalnotify.h"
#include "mgcommon.h"
#include "cgenerateshfile.h"
#define min(a, b) (a > b ? b : a)

class CDlgTiming : public QWidget, public ISignalNotify
{
    Q_OBJECT
public:
    explicit CDlgTiming(QWidget *parent = 0);

    void UpdateTiming();

    void UpdateTimingSettings();
    void UpdateTableTiming();
    void UpdateTableResolution();

    void OnHScroll(int nPos, QScrollBar* pScrollBar);

    //config
    void CheckConfigGroupBox();
    void CheckConfigTabWidget();

    //ISignalNotify
    virtual void onTimingStatuChanged(unsigned long long ullStatusBits);

private:
    QString GetVideoTimingType( int nType );

protected:
    QCheckBox           *m_ckbAutoSampling;
    QCheckBox           *m_ckbAutoAlign;

    QSpinBox            *m_spinPhase;
    QPushButton         *m_btnAutoPhaseAdjust;

    //adjust timing
    QLabel              *m_lblTimingInfo;
    QPushButton         *m_btnPreferred;
    QScrollBar          *m_barHActive;
    QLabel              *m_lblHActive;
    QScrollBar          *m_barHTotal;
    QLabel              *m_lblHTotal;
    QScrollBar          *m_barHOffset;
    QLabel              *m_lblHOffset;
    QScrollBar          *m_barVActive;
    QLabel              *m_lblVActive;
    QScrollBar          *m_barVOffset;
    QLabel              *m_lblVOffset;
    QScrollBar          *m_barClampPos;
    QLabel              *m_lblClampPos;
    QSpinBox            *m_spinAspectX;
    QSpinBox            *m_spinAspectY;
    QPushButton         *m_btnResetTiming;
    QPushButton         *m_btnSaveTiming;

    //Custom timings
    CTableWidget        *m_tableTimings;
    QPushButton         *m_btnDeleteTimings;
    bool                 m_bDeleteFlag;

    //Custom resolutions
    CTableWidget        *m_tableResolutions;
    QPushButton         *m_btnAdd;
    QPushButton         *m_btnModify;
    QPushButton         *m_btnDelete;


    QList<QAction* >    m_listPreferred;
    int                 m_nSelTableTiming;
    int                 m_nSelTableResolution;

    CGenerateSHFile     *m_shFile;

signals:
    void sigTimingStatuChanged(unsigned long long);

public slots:
    void onAutoSamplingChecked(int nStatus);
    void onAutoHAlignChecked(int nStatus);

    void onPhaseValueChanged(int nValue);
    void onAutoPhaseAdjustClicked(bool bFlag);

    void onPreferredTimingClicked(bool bFlag);

    void onHActiveChange(int nValue);
    void onHTotalChange(int nValue);
    void onHOffsetChange(int nValue);
    void onVActiveChange(int nValue);
    void onVOffsetChange(int nValue);
    void onClampPosChange(int nValue);

    void onAspectXValueChanged(int nValue);
    void onAspectYValueChanged(int nValue);

    BOOLEAN IsEqualSyncInfo(const MWCAP_VIDEO_SYNC_INFO * pSyncInfoCurr, const MWCAP_VIDEO_SYNC_INFO * pSyncInfo);
    bool AddCustomTiming(const MWCAP_VIDEO_CUSTOM_TIMING * pTiming);

    void onResetTimingClicked(bool bFlag);
    void onSaveTimingClicked(bool bFlag);
    void onTimingsDeleteClicked(bool bFlag);

    void onResolutionsAddClicked(bool bFlag);
    void onResolutionsModifyClicked(bool bFlag);
    void onResolutionsDeleteClicked(bool bFlag);

    void onTableResolutionsItemClicked();
    void onTableTimingsItemClicked();

    void onPreferredItemClicked();

    void onTimingStatuChangedMain(unsigned long long ullStatusBits);

    void onFocusInResolution(bool bInFocus);
    void onFocusInTiming(bool bInFocus);

};

#endif // CDLGTIMING_H
