#ifndef CDLGVIDEO_H
#define CDLGVIDEO_H

#include <QWidget>
#include "publicheaders.h"
#include "cvideochannel.h"
#include <QApplication>
#include "isignalnotify.h"
#include "cgenerateshfile.h"


class CDlgVideo : public QWidget, public ISignalNotify
{
    Q_OBJECT
public:
    explicit CDlgVideo(QWidget *parent = 0);

    void UpdateVideo();
    void UpdateVideoSettings();
    void UpdateInputSettings();
    void UpdateOutputSettings();

    //ISignalNotify
    virtual void onInputSourceChanged(unsigned long long ullStatusBits);

    //Config file
    void CheckConfigFile();

protected:
    QGroupBox           *m_gpboxInput;
    QGroupBox           *m_gpboxOutput;

    //input
    bool                m_bAspectDefault[16];
    bool                m_bColorfmtDefault[16];
    bool                m_bQuantDefault[16];
    QCheckBox           *m_ckbInputAspect;
    QCheckBox           *m_ckbInputColorFmt;
    QCheckBox           *m_ckbInputQuant;
    QSpinBox            *m_spinInputAspectX;
    QSpinBox            *m_spinInputAspectY;
    QComboBox           *m_cmbInputColorFmt;
    QComboBox           *m_cmbInputQuant;

    //output
    QCheckBox           *m_ckbOutputAspect;
    QCheckBox           *m_ckbOutputColorFmt;
    QCheckBox           *m_ckbOutputCrop;
    QCheckBox           *m_ckbOutputQuant;
    QCheckBox           *m_ckbOutputSaturation;
    QCheckBox           *m_ckbOutputLowlatency;

    QSpinBox            *m_spinOutputAspectX;
    QSpinBox            *m_spinOutputAspectY;
    QSpinBox            *m_spinOutputCropLeft;
    QSpinBox            *m_spinOutputCropRight;
    QSpinBox            *m_spinOutputCropTop;
    QSpinBox            *m_spinOutputCropBottom;

    QComboBox           *m_cmbOutputColorFmt;
    QComboBox           *m_cmbOutputQuant;
    QComboBox           *m_cmbOutputSaturation;
    QComboBox           *m_cmbOutputDeint;
    QComboBox           *m_cmbOutputAR;

    QPushButton         *m_btnLoadPresent;
    QPushButton         *m_btnSavePresent;

    CGenerateSHFile                 *m_shFile;
    MWCAP_VIDEO_PROCESS_SETTINGS    m_videoSettings[16];

signals:
    void sigInputSourceChanged(unsigned long long);

public slots:
    //input
    void onInputAspectChecked(int nStatus);
    void onInputColorFormatChecked(int nStatus);
    void onInputQuantChecked(int nStatus);

    void onInputAspectXValueChanged(int nValue);
    void onInputAspectYValueChanged(int nValue);

    //currentIndexChanged
    void onInputColorFormatIndexChanged(int nIndex);
    void onInputQuantIndexChanged(int nIndex);

    //output
    void onOutputAspectChecked(int nStatus);
    void onLowLatencyChecked(int nStatus);
    void onOutputCropChecked(int nStatus);
    void onOutputColorFormatChecked(int nStatus);
    void onOutputQuantChecked(int nStatus);
    void onOutputSaturationChecked(int nStatus);

    void onOutputAspectXValueChanged(int nValue);
    void onOutputAspectYValueChanged(int nValue);

    void onOutputCropLeft(int nValue);
    void onOutputCropTop(int nValue);
    void onOutputCropRight(int nValue);
    void onOutputCropBottom(int nValue);

    void onOutputColorFormatIndexChanged(int nIndex);
    void onOutputQuantIndexChanged(int nIndex);
    void onOutSaturationIndexChanged(int nIndex);
    void onOutputDeintIndexChanged(int nIndex);
    void onOutputARIndexChanged(int nIndex);

    void onLoadPresetClicked(bool bFlag);
    void onSavePresetClicked(bool bFlag);

    void onInputSourceChangedMain(unsigned long long ullStatusBits);


};

#endif // CDLGVIDEO_H
