#ifndef CVIDEOCHANNEL_H
#define CVIDEOCHANNEL_H

#include <QWidget>
#include "publicheaders.h"
#include "lib-mw-capture.h"
#include "mw-fourcc.h"
#include "videorenderxcb.h"
#include "isignalnotify.h"

#define NUM_PARTIAL_LINE 128
typedef std::vector<ISignalNotify*> CNotifyArray;

class ICaptureCallback
{
public:
    virtual void OnCaptureCallback(VideoRenderImageBuf *pBuffer) = 0;
    virtual void OnWndViewCallback(bool bUpdate) = 0;
};
class CVideochannel
{
//    Q_OBJECT
public:
    explicit CVideochannel(/*QWidget *parent = 0*/);

    bool OpenVideoChannel(int nIndex);
    bool Init();
    void Exit();
    bool StartCapture();
    void StopCapture();
    void SetCallback(VideoRenderImageBuf *m_pVideoBuffer, ICaptureCallback *pCallback);

    QString GetDeviceName(int nIndex);
    int GetVideoHandle()
    {
        return m_hCurrentChannel;
    }
    void SetVideoCaptureFourcc(DWORD dwFourcc)
    {
         m_dwFourcc = dwFourcc;
    }

    //Amp Proc
    void SetVideoBrightness(int nValue);
    void SetVideoHue(int nValue);
    void SetVideoSaturation(int nValue);
    void SetVideoContract(int nValue);

    void SetAmpBrightness(int nValue);
    void SetAmpHue(int nValue);
    void SetAmpSaturation(int nValue);
    void SetAmpContract(int nValue);

    //video
    void SetVideoAspectX(int nAspectX);
    void SetVideoAspectY(int nAspectY);
    void SetLowLatency(bool bLowLatency);
    void SetVideoCropRect(RECT* rectSrc);
    void SetVideoColorFormat(MWCAP_VIDEO_COLOR_FORMAT colorFormat);
    void SetVideoQuant(MWCAP_VIDEO_QUANTIZATION_RANGE quantRange);
    void SetVideoSaturation(MWCAP_VIDEO_SATURATION_RANGE satRange);
    void SetVideoDeintMode(MWCAP_VIDEO_DEINTERLACE_MODE deinterlaceMode);
    void SetVideoARMode(MWCAP_VIDEO_ASPECT_RATIO_CONVERT_MODE aspectRatioConvertMode);

    //osd
    bool UploadOSDImage(unsigned char* pngimage, WORD pngwidth, WORD pngheight);
    void SetOSDImage(char *szPNGImagePath, bool bEnable);
    void SetEnableOSD(bool bImageOSD);

    //v4l2
    void GetOSDImage(char *szPNGImagePath, bool *bEnable);
    void GetAMPBrightness(int *pnBrightness);
    void GetAMPContrast(int *pnContrast);
    void GetAMPHue(int *pnHue);
    void GetAMPSaturation(int *pnSaturation);

    void AddNotify(ISignalNotify* pNotify) {
        m_arrNotify.push_back(pNotify);
    }
    void DelNotify(ISignalNotify* pNotify) {
    }

    //config file
    QString GetCurrentDevicePath()
    {
        return m_strCurrentPath;
    }
    int GetCurrentDeviceIndex()
    {
        return m_nCurrentDeviceIndex;
    }

protected:
    static void *ThreadCapture(void *pParam)
    {
        CVideochannel *pThis = (CVideochannel *)pParam;
        pThis->DoVideoCapture();
    }

    bool DoVideoCapture();


protected:
    pthread_t               m_idCapture;
    bool                    m_bOpenFlag;
    HANDLE64                m_hChannel[16];
    HANDLE64                m_hCurrentChannel;
    QString                 m_strDevicePath[16];
    QString                 m_strCurrentPath;
    int                     m_nCurrentDeviceIndex;

    //capture
    int                   m_cx;
    int                   m_cy;
    bool                  m_bCapture;
    DWORD                 m_dwFourcc;

    VideoRenderImageBuf     *m_Imagebuf;
    ICaptureCallback        *m_Callback;

    //Amp Proc
    int                     m_nBrightness;
    int                     m_nHue;
    int                     m_nSaturation;
    int                     m_nContract;

    //video
    int										m_nAspectX;
    int										m_nAspectY;
    bool                                    m_bLowLatency;
    RECT                                    *m_pRectSrc;
    MWCAP_VIDEO_COLOR_FORMAT				m_colorFormat;
    MWCAP_VIDEO_QUANTIZATION_RANGE			m_quantRange;
    MWCAP_VIDEO_SATURATION_RANGE            m_satRange;
    MWCAP_VIDEO_DEINTERLACE_MODE            m_deinterlaceMode;
    MWCAP_VIDEO_ASPECT_RATIO_CONVERT_MODE	m_aspectRatioConvertMode;

    //osd
    HANDLE64                                m_hImageOSD;
    bool                                    m_bImageOSD;
    CNotifyArray                            m_arrNotify;


signals:

public slots:
};

extern CVideochannel g_capture;

#endif // CVIDEOCHANNEL_H
